#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks
short_description: Resource module for networks
description:
- Manage operations create, update and delete of the resource networks.
- Create a network.
- Delete a network.
- Update a network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  copyFromNetworkId:
    description: The ID of the network to copy configuration from. Other provided parameters
      will override the copied configuration, except type which must match this network's
      type exactly.
    type: str
  enrollmentString:
    description: A unique identifier which can be used for device enrollment or easy
      access through the Meraki SM Registration page or the Self Service Portal. Please
      note that changing this field may cause existing bookmarks to break.
    type: str
  name:
    description: The name of the network.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  notes:
    description: Add any notes or additional information about this network here.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  productTypes:
    description: The product type(s) of the new network. If more than one type is included,
      the network will be a combined network.
    elements: str
    type: list
  tags:
    description: A list of tags to be applied to the network.
    elements: str
    type: list
  timeZone:
    description: The timezone of the network. For a list of allowed timezones, please
      see the 'TZ' column in the table in <a target='_blank' href='https //en.wikipedia.org/wiki/List_of_tz_databas...
      article.</a>.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for organizations,networks createOrganizationNetwork
  description: Complete reference of the createOrganizationNetwork API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-organization-network
- name: Cisco Meraki documentation for organizations,networks deleteNetwork
  description: Complete reference of the deleteNetwork API.
  link: https://developer.cisco.com/meraki/api-v1/#!delete-network
- name: Cisco Meraki documentation for organizations,networks updateNetwork
  description: Complete reference of the updateNetwork API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network
notes:
  - SDK Method used are
    organizationsnetworks.Organizationsnetworks.create_organization_network,
    organizationsnetworks.Organizationsnetworks.delete_network,
    organizationsnetworks.Organizationsnetworks.update_network,

  - Paths used are
    post /organizations/{organizationId}/networks,
    delete /networks/{networkId},
    put /networks/{networkId},
"""

EXAMPLES = r"""
- name: Update by id
  cisco.meraki.networks:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    enrollmentString: my-enrollment-string
    name: Main Office
    networkId: string
    notes: Additional description of the network
    tags:
    - tag1
    - tag2
    timeZone: America/Los_Angeles

- name: Delete by id
  cisco.meraki.networks:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: absent
    networkId: string

- name: Create
  cisco.meraki.networks:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    copyFromNetworkId: N_24329156
    name: Main Office
    notes: Additional description of the network
    organizationId: string
    productTypes:
    - appliance
    - switch
    - wireless
    tags:
    - tag1
    - tag2
    timeZone: America/Los_Angeles

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "organizationId": "string",
      "name": "string",
      "productTypes": [
        "string"
      ],
      "timeZone": "string",
      "tags": [
        "string"
      ],
      "enrollmentString": "string",
      "url": "string",
      "notes": "string",
      "isBoundToConfigTemplate": true
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: client_enrichment_details_info
short_description: Information module for Client Enrichment Details
description:
- Get all Client Enrichment Details.
- >
   Enriches a given network End User context a network user-id or end user's device Mac Address with details about
   the user, the devices that the user is connected to and the assurance issues that the user is impacted by.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Clients GetClientEnrichmentDetails
  description: Complete reference of the GetClientEnrichmentDetails API.
  link: https://developer.cisco.com/docs/dna-center/#!get-client-enrichment-details
notes:
  - SDK Method used are
    clients.Clients.get_client_enrichment_details,

  - Paths used are
    get /dna/intent/api/v1/client-enrichment-details,

"""

EXAMPLES = r"""
- name: Get all Client Enrichment Details
  cisco.dnac.client_enrichment_details_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "userDetails": {
          "id": "string",
          "connectionStatus": "string",
          "hostType": "string",
          "userId": "string",
          "hostName": {},
          "hostOs": {},
          "hostVersion": {},
          "subType": {},
          "lastUpdated": 0,
          "healthScore": [
            {
              "healthType": "string",
              "reason": "string",
              "score": 0
            }
          ],
          "hostMac": "string",
          "hostIpV4": "string",
          "hostIpV6": [
            {}
          ],
          "authType": {},
          "vlanId": "string",
          "ssid": {},
          "location": {},
          "clientConnection": "string",
          "connectedDevice": [
            {}
          ],
          "issueCount": 0,
          "rssi": {},
          "snr": {},
          "dataRate": {},
          "port": {}
        },
        "connectedDevice": [
          {
            "deviceDetails": {
              "family": "string",
              "type": "string",
              "location": {},
              "errorCode": "string",
              "macAddress": "string",
              "role": "string",
              "apManagerInterfaceIp": "string",
              "associatedWlcIp": "string",
              "bootDateTime": {},
              "collectionStatus": "string",
              "interfaceCount": {},
              "lineCardCount": {},
              "lineCardId": {},
              "managementIpAddress": "string",
              "memorySize": "string",
              "platformId": "string",
              "reachabilityFailureReason": "string",
              "reachabilityStatus": "string",
              "snmpContact": "string",
              "snmpLocation": "string",
              "tunnelUdpPort": "string",
              "waasDeviceMode": {},
              "series": "string",
              "inventoryStatusDetail": "string",
              "collectionInterval": "string",
              "serialNumber": "string",
              "softwareVersion": "string",
              "roleSource": "string",
              "hostname": "string",
              "upTime": "string",
              "lastUpdateTime": 0,
              "errorDescription": {},
              "locationName": {},
              "tagCount": "string",
              "lastUpdated": "string",
              "instanceUuid": "string",
              "id": "string",
              "neighborTopology": [
                {
                  "nodes": [
                    {
                      "role": "string",
                      "name": "string",
                      "id": "string",
                      "description": "string",
                      "deviceType": {},
                      "platformId": {},
                      "family": {},
                      "ip": {},
                      "softwareVersion": {},
                      "userId": {},
                      "nodeType": {},
                      "radioFrequency": {},
                      "clients": 0,
                      "count": {},
                      "healthScore": {},
                      "level": 0,
                      "fabricGroup": {}
                    }
                  ],
                  "links": [
                    {
                      "source": "string",
                      "linkStatus": "string",
                      "label": [
                        {}
                      ],
                      "target": "string",
                      "id": {},
                      "portUtilization": {}
                    }
                  ]
                }
              ],
              "cisco360view": "string"
            }
          }
        ],
        "issueDetails": {
          "issue": [
            {
              "issueId": "string",
              "issueSource": "string",
              "issueCategory": "string",
              "issueName": "string",
              "issueDescription": "string",
              "issueEntity": "string",
              "issueEntityValue": "string",
              "issueSeverity": "string",
              "issuePriority": "string",
              "issueSummary": "string",
              "issueTimestamp": 0,
              "suggestedActions": [
                {
                  "message": "string",
                  "steps": [
                    {}
                  ]
                }
              ],
              "impactedHosts": [
                {
                  "hostType": "string",
                  "hostName": "string",
                  "hostOs": "string",
                  "ssid": "string",
                  "connectedInterface": "string",
                  "macAddress": "string",
                  "failedAttempts": 0,
                  "location": {
                    "siteId": "string",
                    "siteType": "string",
                    "area": "string",
                    "building": "string",
                    "floor": {},
                    "apsImpacted": [
                      {}
                    ]
                  },
                  "timestamp": 0
                }
              ]
            }
          ]
        }
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r'''
---
module: win_powershell
version_added: 1.5.0
short_description: Run PowerShell scripts
description:
- Runs a PowerShell script and outputs the data in a structured format.
- Use M(ansible.windows.win_command) or M(ansible.windows.win_shell) to run a traditional PowerShell process with
  stdout, stderr, and rc results.
options:
  arguments:
    description:
    - A list of arguments to pass to I(executable) when running a script in another PowerShell process.
    - These are not arguments to pass to I(script), use I(parameters) for that purpose.
    type: list
    elements: str
  chdir:
    description:
    - The PowerShell location to set when starting the script.
    - This can be a location in any of the PowerShell providers.
    - The default location is dependent on many factors, if relative paths are used then set this option.
    type: str
  creates:
    description:
    - A path or path filter pattern; when the referenced path exists on the target host, the task will be skipped.
    type: str
  depth:
    description:
    - How deep the return values are serialized for C(result), C(output), and C(information[x].message_data).
    - This also controls the depth of the diff output set by C($Ansible.Diff).
    - Setting this to a higher value can dramatically increase the amount of data that needs to be returned.
    default: 2
    type: int
  error_action:
    description:
    - The C($ErrorActionPreference) to set before executing I(script).
    - C(silently_continue) will ignore any errors and exceptions raised.
    - C(continue) is the default behaviour in PowerShell, errors are present in the I(error) return value but only
      terminating exceptions will stop the script from continuing and set it as failed.
    - C(stop) will treat errors like exceptions, will stop the script and set it as failed.
    choices:
    - silently_continue
    - continue
    - stop
    default: continue
    type: str
  executable:
    description:
    - A custom PowerShell executable to run the script in.
    - When not defined the script will run in the current module PowerShell interpreter.
    - Both the remote PowerShell and the one specified by I(executable) must be running on PowerShell v5.1 or newer.
    - Setting this value may change the values returned in the C(output) return value depending on the underlying .NET
      type.
    type: str
  parameters:
    description:
    - Parameters to pass into the script as key value pairs.
    - The key corresponds to the parameter name and the value is the value for that parameter.
    type: dict
  removes:
    description:
    - A path or path filter pattern; when the referenced path B(does not) exist on the target host, the task will be
      skipped.
    type: str
  script:
    description:
    - The PowerShell script to run.
    type: str
    required: true
seealso:
- module: ansible.windows.win_command
- module: ansible.windows.win_shell
notes:
- The module is set as failed when a terminating exception is throw, or C(error_action=stop) and a normal error record
  is raised.
- The output values are processed using a custom filter and while it mostly matches the C(ConvertTo-Json) result the
  following value types are different.
- C(DateTime) will be an ISO 8601 string in UTC, C(DateTimeOffset) will have the offset as specified by the value.
- C(Enum) will contain a dictionary with C(Type), C(String), C(Value) being the type name, string representation and
  raw integer value respectively.
- C(Type) will contain a dictionary with C(Name), C(FullName), C(AssemblyQualifiedName), C(BaseType) being the type
  name, the type name including the namespace, the full assembly name the type was defined in and the base type it
  derives from.
- The script has access to the C($Ansible) variable where it can set C(Result), C(Changed), C(Failed), C(Diff),
  or access C(Tmpdir).
- C($Ansible.Result) is a value that is returned back to the controller as is.
- C($Ansible.Diff) was added in the C(1.12.0) release of C(ansible.windows) and is a dictionary that is set to the diff
  result that can be interepreted by Ansible.
- C($Ansible.Changed) can be set to C(true) or C(false) to reflect whether the module made a change or not. By default
  this is set to C(true).
- C($Ansible.Failed) can be set to C(true) if the script wants to return the failure back to the controller.
- C($Ansible.Tmpdir) is the path to a temporary directory to use as a scratch location that is cleaned up after the
  module has finished.
- C($Ansible.Verbosity) reveals Ansible's verbosity level for this play. Allows the script to set VerbosePreference/DebugPreference
  based on verbosity. Added in C(1.9.0).
- Any host/console direct output like C(Write-Host) or C([Console]::WriteLine) is not considered an output object, they are
  returned as a string in I(host_out) and I(host_err).
- Any output stream object is instead returned as a list in I(output). This is true not only for C(Write-Output) and its
  built-in alias C(echo), but also for implicit output; i.e. C(Write-Output "foo") and C("foo") give the same result.
- The module will skip running the script when in check mode unless the script defines
  C([CmdletBinding(SupportsShouldProcess)]).
author:
- Jordan Borean (@jborean93)
'''

EXAMPLES = r'''
- name: Run basic PowerShell script
  ansible.windows.win_powershell:
    script: |
      echo "Hello World"

- name: Run PowerShell script with parameters
  ansible.windows.win_powershell:
    script: |
      [CmdletBinding()]
      param (
          [String]
          $Path,

          [Switch]
          $Force
      )

      New-Item -Path $Path -ItemType Directory -Force:$Force
    parameters:
      Path: C:\temp
      Force: true

- name: Run PowerShell script that modifies the module changed result
  ansible.windows.win_powershell:
    script: |
      if (Get-Service -Name test -ErrorAction SilentlyContinue) {
          Remove-Service -Name test
      }
      else {
          $Ansible.Changed = $false
      }

- name: Run PowerShell script in PowerShell 7
  ansible.windows.win_powershell:
    script: |
      $PSVersionTable.PSVersion.Major
    executable: pwsh.exe
    arguments:
    - -ExecutionPolicy
    - ByPass
  register: pwsh_output
  failed_when:
  - pwsh_output.output[0] != 7

- name: Run code in check mode
  ansible.windows.win_powershell:
    script: |
      [CmdletBinding(SupportsShouldProcess)]
      param ()

      # Use $Ansible to detect check mode
      if ($Ansible.CheckMode) {
          echo 'running in check mode'
      }
      else {
          echo 'running in normal mode'
      }

      # Use builtin ShouldProcess (-WhatIf)
      if ($PSCmdlet.ShouldProcess('target')) {
          echo 'also running in normal mode'
      }
      else {
          echo 'also running in check mode'
      }
  check_mode: yes

- name: Return a failure back to Ansible
  ansible.windows.win_powershell:
    script: |
      if (Test-Path C:\bad.file) {
          $Ansible.Failed = $true
      }

- name: Define when the script made a change or not
  ansible.windows.win_powershell:
    script: |
      if ((Get-Item WSMan:\localhost\Service\Auth\Basic).Value -eq 'true') {
          Set-Item WSMan:\localhost\Service\Auth\Basic -Value false
      }
      else {
          $Ansible.Changed = $true
      }

- name: Define when to enable Verbose/Debug output
  ansible.windows.win_powershell:
    script: |
      if ($Ansible.Verbosity -ge 3) {
          $VerbosePreference = "Continue"
      }
      if ($Ansible.Verbosity -eq 5) {
          $DebugPreference = "Continue"
      }
      Write-Output "Hello World!"
      Write-Verbose "Hello World!"
      Write-Debug "Hello World!"

'''

RETURN = r'''
result:
  description:
  - The values that were set by C($Ansible.Result) in the script.
  - Defaults to an empty dict but can be set to anything by the script.
  returned: always
  type: complex
  sample: {'key': 'value', 'other key': 1}
  contains: {}  # Satisfy the validate-modules sanity check
host_out:
  description:
  - The strings written to the host output, typically the stdout.
  - This is not the same as objects sent to the output stream in PowerShell.
  returned: always
  type: str
  sample: "Line 1\nLine 2"
host_err:
  description:
  - The strings written to the host error output, typically the stderr.
  - This is not the same as objects sent to the error stream in PowerShell.
  returned: always
  type: str
  sample: "Error 1\nError 2"
output:
  description:
  - A list containing all the objects outputted by the script.
  - The list elements can be anything as it is based on what was ran.
  returned: always
  type: list
  sample: ['output 1', 2, ['inner list'], {'key': 'value'}, None]
error:
  description:
  - A list of error records created by the script.
  returned: always
  type: list
  elements: dict
  contains:
    output:
      description:
      - The formatted error record message as typically seen in a PowerShell console.
      type: str
      returned: always
      sample: |
        Write-Error "error" : error
            + CategoryInfo          : NotSpecified: (:) [Write-Error], WriteErrorException
            + FullyQualifiedErrorId : Microsoft.PowerShell.Commands.WriteErrorException
    error_details:
      description:
      - Additional details about an ErrorRecord.
      - Can be null if there are not additional details.
      type: dict
      contains:
        message:
          description:
          - Message for the error record.
          returned: always
          type: str
          sample: Specific error message
        recommended_action:
          description:
          - Recommended action in the even that this error occurs.
          - This is empty unless the code which generates the error adds this explicitly.
          returned: always
          type: str
          sample: Delete file
    exception:
      description:
      - Details about the exception behind the error record.
      type: dict
      contains:
        message:
          description:
          - The exception message.
          type: str
          returned: always
          sample: The method ran into an error
        type:
          description:
          - The full .NET type of the Exception class.
          type: str
          returned: always
          sample: System.Exception
        help_link:
          description:
          - A link to the help details for the exception.
          - May not be set as it's dependent on whether the .NET exception class provides this info.
          type: str
          returned: always
          sample: http://docs.ansible.com/
        source:
          description:
          - Name of the application or object that causes the error.
          - This may be an empty string as it's dependent on the code that raises the exception.
          type: str
          returned: always
          sample: C:\Windows
        hresult:
          description:
          - The signed integer assigned to this exception.
          - May not be set as it's dependent on whether the .NET exception class provides this info.
          type: int
          returned: always
          sample: -1
        inner_exception:
          description:
          - The inner exception details if there is one present.
          - The dict contains the same keys as a normal exception.
          returned: always
          type: dict
    target_object:
      description:
      - The object which the error occured.
      - May be null if no object was specified when the record was created.
      - Type type of this object depends on the error record itself.
      - If the value is a complex type, it will follow the C(depth) limit specified.
      type: raw
      returned: always
      sample: C:\Windows
    category_info:
      description:
      - More information about the error record.
      type: dict
      contains:
        category:
          description:
          - The category name of the error record.
          type: str
          returned: always
          sample: NotSpecified
        category_id:
          description:
          - The integer representation of the category.
          type: int
          returned: always
          sample: 0
        activity:
          description:
          - Description of the operation which encountered the error.
          type: str
          returned: always
          sample: Write-Error
        reason:
          description:
          - Description of the error.
          type: str
          returned: always
          sample: WriteErrorException
        target_name:
          description:
          - Description of the target object.
          - Can be an empty string if no target was specified.
          type: str
          returned: always
          sample: C:\Windows
        target_type:
          description:
          - Description of the type of the target object.
          - Can be an empty string if no target object was specified.
          type: str
          returned: always
          sample: String
    fully_qualified_error_id:
      description:
      - The unique identifier for the error condition
      - May be null if no id was specified when the record was created.
      type: str
      returned: always
      sample: ParameterBindingFailed
    script_stack_trace:
      description:
      - The script stack trace for the error record.
      type: str
      returned: always
      sample: 'at <ScriptBlock>, <No file>: line 1'
    pipeline_iteration_info:
      description:
      - The status of the pipeline when this record was created.
      - The values are 0 index based.
      - Each element entry represents the command index in a pipeline statement.
      - The value of each element represents the pipeline input idx in that command.
      - For Example C('C:\Windows', 'C:\temp' | Get-ChildItem | Get-Item), C([1, 2, 9]) represents an error occured
        with the 2nd output, 3rd, and 9th output of the 1st, 2nd, and 3rd command in that pipeline respectively.
      type: list
      elements: int
      returned: always
      sample: [0, 0]
warning:
  description:
  - A list of warning messages created by the script.
  - Warning messages only appear when C($WarningPreference = 'Continue').
  returned: always
  type: list
  elements: str
  sample: ['warning record']
verbose:
  description:
  - A list of warning messages created by the script.
  - Verbose messages only appear when C($VerbosePreference = 'Continue').
  returned: always
  type: list
  elements: str
  sample: ['verbose record']
debug:
  description:
  - A list of warning messages created by the script.
  - Debug messages only appear when C($DebugPreference = 'Continue').
  returned: always
  type: list
  elements: str
  sample: ['debug record']
information:
  description:
  - A list of information records created by the script.
  - The information stream was only added in PowerShell v5, older versions will always have an empty list as a value.
  returned: always
  type: list
  elements: dict
  contains:
    message_data:
      description:
      - Message data associated with the record.
      - The value here can be of any type.
      type: complex
      returned: always
      sample: information record
      contains: {}  # Satisfy the validate-modules sanity check
    source:
      description:
      - The source of the record.
      type: str
      returned: always
      sample: Write-Information
    time_generated:
      description:
      - The time the record was generated.
      - This is the time in UTC as an ISO 8601 formatted string.
      type: str
      returned: always
      sample: '2021-02-11T04:46:00.4694240Z'
    tags:
      description:
      - A list of tags associated with the record.
      type: list
      elements: str
      returned: always
      sample: ['Host']
'''



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type
DOCUMENTATION = """
module: ios_banner
author: Ricardo Carrillo Cruz (@rcarrillocruz)
short_description: Module to configure multiline banners.
description:
  - This will configure both login and motd banners on remote devices running Cisco
    IOS. It allows playbooks to add or remote banner text from the active running configuration.
version_added: 1.0.0
extends_documentation_fragment:
  - cisco.ios.ios
notes:
  - Tested against Cisco IOSXE Version 17.3 on CML.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
options:
  banner:
    description:
      - Specifies which banner should be configured on the remote device. In Ansible
        2.4 and earlier only I(login) and I(motd) were supported.
    required: true
    choices:
      - login
      - motd
      - exec
      - incoming
      - slip-ppp
    type: str
  multiline_delimiter:
    description:
      - Specify the delimiting character than will be used for configuration.
    default: "@"
    type: str
  text:
    description:
      - The banner text that should be present in the remote device running configuration.  This
        argument accepts a multiline string, with no empty lines. Requires I(state=present).
    type: str
  state:
    description:
      - Specifies whether or not the configuration is present in the current devices
        active running configuration.
    default: present
    type: str
    choices:
      - present
      - absent
"""

EXAMPLES = """
- name: Configure the login banner
  cisco.ios.ios_banner:
    banner: login
    text: |
      this is my login banner
      that contains a multiline
      string
    state: present

- name: Remove the motd banner
  cisco.ios.ios_banner:
    banner: motd
    state: absent

- name: Configure banner from file
  cisco.ios.ios_banner:
    banner: motd
    text: "{{ lookup('file', './config_partial/raw_banner.cfg') }}"  # Use unix formatted text files (LF not CRLF) to avoid idempotency issues.
    state: present

- name: Configure the login banner using delimiter
  cisco.ios.ios_banner:
    banner: login
    multiline_delimiter: x
    text: this is my login banner
    state: present
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always
  type: list
  sample:
    - banner login
    - this is my login banner
    - that contains a multiline
    - string
"""
from re import M, search

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.ios import (
    get_config,
    load_config,
)


def map_obj_to_commands(updates, module):
    commands = list()
    want, have = updates
    state = module.params["state"]
    multiline_delimiter = module.params.get("multiline_delimiter")
    if state == "absent" and "text" in have.keys() and have["text"]:
        commands.append("no banner %s" % module.params["banner"])
    elif state == "present":
        if have.get("text") and len(have.get("text")) > 1:
            haved = have.get("text").rstrip("\n")
        else:
            haved = ""
        if want["text"] and (want["text"].rstrip("\n") != haved):
            banner_cmd = "banner %s" % module.params["banner"]
            banner_cmd += " {0}\n".format(multiline_delimiter)
            banner_cmd += want["text"].strip("\n")
            banner_cmd += "\n{0}".format(multiline_delimiter)
            commands.append(banner_cmd)
    return commands


def map_config_to_obj(module):
    """
    This function gets the banner config without stripping any whitespaces,
    and then fetches the required banner from it.
    :param module:
    :return: banner config dict object.
    """
    out = get_config(module, flags="| begin banner %s" % module.params["banner"])
    if out:
        regex = search("banner " + module.params["banner"] + " \\^C{1,}\n", out, M)
        if regex:
            regex = regex.group()
            output = str((out.split(regex))[1].split("^C\n")[0])
        else:
            output = None
    else:
        output = None
    obj = {"banner": module.params["banner"], "state": "absent"}
    if output:
        obj["text"] = output
        obj["state"] = "present"
    return obj


def map_params_to_obj(module):
    text = module.params["text"]
    return {
        "banner": module.params["banner"],
        "text": text,
        "multiline_delimiter": module.params["multiline_delimiter"],
        "state": module.params["state"],
    }


def main():
    """main entry point for module execution"""
    argument_spec = dict(
        banner=dict(required=True, choices=["login", "motd", "exec", "incoming", "slip-ppp"]),
        multiline_delimiter=dict(default="@"),
        text=dict(),
        state=dict(default="present", choices=["present", "absent"]),
    )
    required_if = [("state", "present", ("text",))]
    module = AnsibleModule(
        argument_spec=argument_spec,
        required_if=required_if,
        supports_check_mode=True,
    )
    warnings = list()
    result = {"changed": False}
    if warnings:
        result["warnings"] = warnings
    want = map_params_to_obj(module)
    have = map_config_to_obj(module)
    commands = map_obj_to_commands((want, have), module)
    result["commands"] = commands
    if commands:
        if not module.check_mode:
            load_config(module, commands)
        result["changed"] = True
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_wireless_latency_history_info
short_description: Information module for networks _wireless _latencyhistory
description:
- Get all networks _wireless _latencyhistory.
- Return average wireless latency over time for a network, device, or network client.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
    - NetworkId path parameter. Network ID.
    type: str
  t0:
    description:
    - T0 query parameter. The beginning of the timespan for the data. The maximum lookback period is 31 days from today.
    type: str
  t1:
    description:
    - T1 query parameter. The end of the timespan for the data. T1 can be a maximum of 31 days after t0.
    type: str
  timespan:
    description:
    - >
      Timespan query parameter. The timespan for which the information will be fetched. If specifying timespan, do
      not specify parameters t0 and t1. The value must be in seconds and be less than or equal to 31 days. The
      default is 7 days.
    type: float
  resolution:
    description:
    - >
      Resolution query parameter. The time resolution in seconds for returned data. The valid resolutions are 300,
      600, 1200, 3600, 14400, 86400. The default is 86400.
    type: int
  autoResolution:
    description:
    - >
      AutoResolution query parameter. Automatically select a data resolution based on the given timespan; this
      overrides the value specified by the 'resolution' parameter. The default setting is false.
    type: bool
  clientId:
    description:
    - ClientId query parameter. Filter results by network client.
    type: str
  deviceSerial:
    description:
    - DeviceSerial query parameter. Filter results by device.
    type: str
  apTag:
    description:
    - ApTag query parameter. Filter results by AP tag.
    type: str
  band:
    description:
    - Band query parameter. Filter results by band (either '2.4', '5' or '6').
    type: str
  ssid:
    description:
    - Ssid query parameter. Filter results by SSID number.
    type: int
  accessCategory:
    description:
    - AccessCategory query parameter. Filter by access category.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for wireless getNetworkWirelessLatencyHistory
  description: Complete reference of the getNetworkWirelessLatencyHistory API.
  link: https://developer.cisco.com/meraki/api-v1/#!get-network-wireless-latency-history
notes:
  - SDK Method used are
    wireless.Wireless.get_network_wireless_latency_history,

  - Paths used are
    get /networks/{networkId}/wireless/latencyHistory,
"""

EXAMPLES = r"""
- name: Get all networks _wireless _latencyhistory
  cisco.meraki.networks_wireless_latency_history_info:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    t0: string
    t1: string
    timespan: 0
    resolution: 0
    autoResolution: True
    clientId: string
    deviceSerial: string
    apTag: string
    band: string
    ssid: 0
    accessCategory: string
    networkId: string
  register: result

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "startTs": "string",
        "endTs": "string",
        "avgLatencyMs": 0
      }
    ]
"""



#!/usr/bin/python
#
# Copyright (c) 2022 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_aksagentpool
version_added: '1.14.0'
short_description: Manage node pools in Kubernetes kubernetes cluster
description:
    - Create, update or delete node pools in kubernetes cluster.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: True
    cluster_name:
        description:
            - The name of the kubernetes cluster.
        type: str
        required: True
    name:
        description:
            - The name of the node agent pool.
        type: str
        required: True
    count:
        description:
            - Number of agents (VMs) to host docker containers.
        type: int
    vm_size:
        description:
            - Size of agent VMs
        type: str
    os_disk_size_gb:
        description:
            - OS Disk Size in GB to be used to specify the disk size for every machine in this master/agent pool.
        type: int
    vnet_subnet_id:
        description:
            - VNet SubnetID specifies the VNet's subnet identifier.
        type: str
    availability_zones:
        description:
            - Availability zones for nodes. Must use VirtualMachineScaleSets AgentPoolType.
        type: list
        elements: int
        choices:
            - 1
            - 2
            - 3
    os_type:
        description:
            - OsType to be used to specify os type.
        type: str
        choices:
            - Linux
            - Windows
    orchestrator_version:
        description:
            - Version of orchestrator specified when creating the managed cluster.
        type: str
    type_properties_type:
        description:
            - AgentPoolType represents types of an agent pool.
        type: str
        choices:
            - VirtualMachineScaleSets
            - AvailabilitySet
    mode:
        description:
            - AgentPoolMode represents mode of an agent pool.
        type: str
        choices:
            - System
            - User
    enable_auto_scaling:
        description:
            - Whether to enable auto-scaler.
        type: bool
    max_count:
        description:
            - Maximum number of nodes for auto-scaling.
        type: int
    node_labels:
        description:
            -  Agent pool node labels to be persisted across all nodes in agent pool.
        type: dict
    min_count:
        description:
            - Minimum number of nodes for auto-scaling.
        type: int
    max_pods:
        description:
            - Maximum number of pods that can run on a node.
        type: int
    state:
        description:
            - State of the automation runbook. Use C(present) to create or update a automation runbook and use C(absent) to delete.
        type: str
        default: present
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Add new node agent pool
  azure_rm_aksagentpool:
    resource_group: "{{ resource_group }}"
    cluster_name: aksfred01
    name: default-new
    count: 2
    vm_size: Standard_B2s
    type_properties_type: VirtualMachineScaleSets
    mode: System
    node_labels: {"release":"stable"}
    max_pods: 42
    orchestrator_version: 1.23.5
    availability_zones:
      - 1
      - 2
- name: Delete node agent pool
  azure_rm_aksagentpool:
    resource_group: "{{ resource_group }}"
    cluster_name: aksfred01
    name: default-new
'''

RETURN = '''
aks_agent_pools:
    description:
        - Details for a node pool in the managed Kubernetes cluster.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxxf/resourcegroups/myRG/providers/Microsoft.ContainerService/managedClusters/cluster/agentPools/default"
        resource_group:
            description:
                - Resource group name.
            type: str
            returned: always
            sample: myRG
        name:
            description:
                - Resource name.
            type: str
            returned: always
            sample: default
        cluster_name:
            description:
                - The cluster name.
            type: str
            returned: always
            sample: testcluster
        availability_zones:
            description:
                - Availability zones for nodes. Must use VirtualMachineScaleSets AgentPoolType.
            type: list
            returned: always
            sample: [1, 2]
        count:
            description:
                - Number of agents (VMs) to host docker containers.
            type: int
            returned: always
            sample: 2
        enable_auto_scaling:
            description:
                - Whether to enable auto-scaler.
            type: str
            returned: always
            sample: null
        enable_node_public_ip:
            description:
                -  Enable public IP for nodes.
            type: str
            returned: always
            sample: bool
        max_count:
            description:
                - Maximum number of nodes for auto-scaling.
            type: int
            returned: always
            sample: 10
        min_count:
            description:
                - Minimum number of nodes for auto-scaling.
            type: int
            returned: always
            sample: 1
        max_pods:
            description:
                - Maximum number of pods that can run on a node.
            type: int
            returned: always
            sample: 42
        mode:
            description:
                - AgentPoolMode represents mode of an agent pool.
            type: str
            returned: always
            sample: System
        node_image_version:
            description:
                - Version of node image.
            type: str
            returned: always
            sample: AKSUbuntu-1804gen2containerd-2022.08.23
        node_labels:
            description:
                - Agent pool node labels to be persisted across all nodes in agent pool.
            type: list
            returned: always
            sample: ["release": "stable"]
        node_taints:
            description:
                - Taints added to new nodes during node pool create and scale.
            type: str
            returned: always
            sample: null
        orchestrator_version:
            description:
                - Version of orchestrator specified when creating the managed cluster.
            type: str
            returned: always
            sample: 1.22.11
        os_disk_size_gb:
            description:
                - OS Disk Size in GB to be used to specify the disk size for every machine in this master/agent pool.
            type: int
            returned: always
            sample: 128
        os_type:
            description:
                - OsType to be used to specify os type.
            type: str
            returned: always
            sample: Linux
        provisioning_state:
            description:
                - The current deployment or provisioning state, which only appears in the response.
            type: str
            returned: always
            sample: Succeeded
        scale_set_eviction_policy:
            description:
                - ScaleSetEvictionPolicy to be used to specify eviction policy for Spot virtual machine scale set.
            type: str
            returned: always
            sample: null
        scale_set_priority:
            description:
                - caleSetPriority to be used to specify virtual machine scale set priority.
            type: str
            returned: always
            sample: null
        spot_max_price:
            description:
                - SpotMaxPrice to be used to specify the maximum price you are willing to pay in US Dollars.
            type: float
            returned: always
            sample: null
        type:
            description:
                - Resource Type.
            type: str
            returned: always
            sample: Microsoft.ContainerService/managedClusters/agentPools
        type_properties_type:
            description:
                - AgentPoolType represents types of an agent pool.
            type: str
            returned: always
            sample: VirtualMachineScaleSets
        upgrade_settings:
            description:
                - Settings for upgrading the agentpool.
            type: str
            returned: always
            sample: null
        vm_size:
            description:
                - Size of agent VMs.
            type: str
            returned: always
            sample: Standard_B2s
        vnet_subnet_id:
            description:
                - VNet SubnetID specifies the VNet's subnet identifier.
            type: str
            returned: always
            sample: null
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    pass


class AzureRMAksAgentPool(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            cluster_name=dict(
                type='str',
                required=True
            ),
            count=dict(
                type='int',
            ),
            vm_size=dict(
                type='str',
            ),
            os_disk_size_gb=dict(
                type='int'
            ),
            vnet_subnet_id=dict(
                type='str'
            ),
            availability_zones=dict(
                type='list',
                elements='int',
                choices=[1, 2, 3]
            ),
            os_type=dict(
                type='str',
                choices=['Linux', 'Windows']
            ),
            orchestrator_version=dict(
                type='str',
            ),
            type_properties_type=dict(
                type='str',
                choices=['VirtualMachineScaleSets', 'AvailabilitySet']
            ),
            mode=dict(
                type='str',
                choices=['System', 'User'],
            ),
            enable_auto_scaling=dict(
                type='bool'
            ),
            max_count=dict(
                type='int'
            ),
            node_labels=dict(
                type='dict'
            ),
            min_count=dict(
                type='int'
            ),
            max_pods=dict(
                type='int'
            ),
            state=dict(
                type='str',
                choices=['present', 'absent'],
                default='present'
            )
        )
        # store the results of the module operation
        self.results = dict()
        self.resource_group = None
        self.name = None
        self.cluster_name = None
        self.count = None
        self.vm_size = None
        self.mode = None
        self.os_disk_size_gb = None
        self.storage_profiles = None
        self.vnet_subnet_id = None
        self.availability_zones = None
        self.os_type = None
        self.orchestrator_version = None
        self.type_properties_type = None
        self.enable_auto_scaling = None
        self.max_count = None
        self.node_labels = None
        self.min_count = None
        self.max_pods = None
        self.body = dict()

        super(AzureRMAksAgentPool, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec):
            setattr(self, key, kwargs[key])
            if key not in ['resource_group', 'cluster_name', 'name', 'state']:
                self.body[key] = kwargs[key]

        agent_pool = self.get()
        changed = False
        response = None

        if self.state == 'present':
            if agent_pool:
                for key in self.body.keys():
                    if self.body[key] is not None and self.body[key] != agent_pool[key]:
                        changed = True
                    else:
                        self.body[key] = agent_pool[key]
            else:
                changed = True

            if changed:
                if not self.check_mode:
                    response = self.create_or_update(self.body)

        else:
            if not self.check_mode:
                if agent_pool:
                    response = self.delete_agentpool()
                    changed = True
                else:
                    changed = False
            else:
                changed = True

        self.results['changed'] = changed
        self.results['aks_agent_pools'] = response
        return self.results

    def get(self):
        try:
            response = self.managedcluster_client.agent_pools.get(self.resource_group, self.cluster_name, self.name)
            return self.to_dict(response)
        except ResourceNotFoundError:
            pass

    def create_or_update(self, parameters):
        try:
            response = self.managedcluster_client.agent_pools.begin_create_or_update(self.resource_group, self.cluster_name, self.name, parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
            return self.to_dict(response)
        except Exception as exc:
            self.fail('Error when creating cluster node agent pool {0}: {1}'.format(self.name, exc))

    def delete_agentpool(self):
        try:
            response = self.managedcluster_client.agent_pools.begin_delete(self.resource_group, self.cluster_name, self.name)
        except Exception as exc:
            self.fail('Error when deleting cluster agent pool {0}: {1}'.format(self.name, exc))

    def to_dict(self, agent_pool):
        if not agent_pool:
            return None
        agent_pool_dict = dict(
            resource_group=self.resource_group,
            cluster_name=self.cluster_name,
            id=agent_pool.id,
            type=agent_pool.type,
            name=agent_pool.name,
            count=agent_pool.count,
            vm_size=agent_pool.vm_size,
            os_disk_size_gb=agent_pool.os_disk_size_gb,
            vnet_subnet_id=agent_pool.vnet_subnet_id,
            max_pods=agent_pool.max_pods,
            os_type=agent_pool.os_type,
            max_count=agent_pool.max_count,
            min_count=agent_pool.min_count,
            enable_auto_scaling=agent_pool.enable_auto_scaling,
            type_properties_type=agent_pool.type_properties_type,
            mode=agent_pool.mode,
            orchestrator_version=agent_pool.orchestrator_version,
            node_image_version=agent_pool.node_image_version,
            upgrade_settings=dict(),
            provisioning_state=agent_pool.provisioning_state,
            availability_zones=[],
            enable_node_public_ip=agent_pool.enable_node_public_ip,
            scale_set_priority=agent_pool.scale_set_priority,
            scale_set_eviction_policy=agent_pool.scale_set_eviction_policy,
            spot_max_price=agent_pool.spot_max_price,
            node_labels=agent_pool.node_labels,
            node_taints=agent_pool.node_taints,
        )

        if agent_pool.upgrade_settings is not None:
            agent_pool_dict['upgrade_settings']['max_surge'] = agent_pool.upgrade_settings.max_surge

        if agent_pool.availability_zones is not None:
            for key in agent_pool.availability_zones:
                agent_pool_dict['availability_zones'].append(int(key))

        return agent_pool_dict


def main():
    AzureRMAksAgentPool()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: api_gateway
version_added: 1.0.0
short_description: Manage AWS API Gateway APIs
description:
  - Allows for the management of API Gateway APIs.
  - Normally you should give the api_id since there is no other
    stable guaranteed unique identifier for the API.  If you do
    not give api_id then a new API will be created each time
    this is run.
  - swagger_file and swagger_text are passed directly on to AWS
    transparently whilst swagger_dict is an ansible dict which is
    converted to JSON before the API definitions are uploaded.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_api_gateway).
    The usage did not change.
options:
  api_id:
    description:
      - The ID of the API you want to manage.
    type: str
  state:
    description: Create or delete API Gateway.
    default: present
    choices: [ 'present', 'absent' ]
    type: str
  swagger_file:
    description:
      - JSON or YAML file containing swagger definitions for API.
        Exactly one of I(swagger_file), I(swagger_text) or I(swagger_dict) must
        be present.
    type: path
    aliases: ['src', 'api_file']
  swagger_text:
    description:
      - Swagger definitions for API in JSON or YAML as a string direct
        from playbook.
    type: str
  swagger_dict:
    description:
      - Swagger definitions API ansible dictionary which will be
        converted to JSON and uploaded.
    type: json
  stage:
    description:
      - The name of the stage the API should be deployed to.
    type: str
  deploy_desc:
    description:
      - Description of the deployment.
      - Recorded and visible in the AWS console.
    default: Automatic deployment by Ansible.
    type: str
  cache_enabled:
    description:
      - Enable API GW caching of backend responses.
    type: bool
    default: false
  cache_size:
    description:
      - Size in GB of the API GW cache, becomes effective when cache_enabled is true.
    choices: ['0.5', '1.6', '6.1', '13.5', '28.4', '58.2', '118', '237']
    type: str
    default: '0.5'
  stage_variables:
    description:
      - ENV variables for the stage. Define a dict of key values pairs for variables.
    type: dict
    default: {}
  stage_canary_settings:
    description:
      - Canary settings for the deployment of the stage.
      - 'Dict with following settings:'
      - 'C(percentTraffic): The percent (0-100) of traffic diverted to a canary deployment.'
      - 'C(deploymentId): The ID of the canary deployment.'
      - 'C(stageVariableOverrides): Stage variables overridden for a canary release deployment.'
      - 'C(useStageCache): A Boolean flag to indicate whether the canary deployment uses the stage cache or not.'
      - See docs U(https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/apigateway.html#APIGateway.Client.create_stage)
    type: dict
    default: {}
  tracing_enabled:
    description:
      - Specifies whether active tracing with X-ray is enabled for the API GW stage.
    type: bool
    default: false
  endpoint_type:
    description:
      - Type of endpoint configuration.
      - Use C(EDGE) for an edge optimized API endpoint,
        C(REGIONAL) for just a regional deploy or C(PRIVATE) for a private API.
      - This flag will only be used when creating a new API Gateway setup, not for updates.
    choices: ['EDGE', 'REGIONAL', 'PRIVATE']
    type: str
    default: EDGE
  name:
    description:
      - The name of the RestApi.
    type: str
    version_added: 6.2.0
  lookup:
    description:
      - Look up API gateway by either I(tags) (and I(name) if supplied) or by I(api_id).
      - If I(lookup=tag) and I(tags) is not specified then no lookup for an existing API gateway
        is performed and a new API gateway will be created.
      - When using I(lookup=tag), multiple matches being found will result in a failure and no changes will be made.
      - To change the tags of a API gateway use I(lookup=id).
    default: tag
    choices: [ 'tag', 'id' ]
    type: str
    version_added: 6.2.0
author:
  - 'Michael De La Rue (@mikedlr)'
notes:
  - 'Tags are used to uniquely identify API gateway when the I(api_id) is not supplied. version_added=6.2.0'
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
  - amazon.aws.tags
"""

EXAMPLES = r"""
- name: Setup AWS API Gateway setup on AWS and deploy API definition
  community.aws.api_gateway:
    swagger_file: my_api.yml
    stage: production
    cache_enabled: true
    cache_size: '1.6'
    tracing_enabled: true
    endpoint_type: EDGE
    state: present

- name: Update API definition to deploy new version
  community.aws.api_gateway:
    api_id: 'abc123321cba'
    swagger_file: my_api.yml
    deploy_desc: Make auth fix available.
    cache_enabled: true
    cache_size: '1.6'
    endpoint_type: EDGE
    state: present

- name: Update API definitions and settings and deploy as canary
  community.aws.api_gateway:
    api_id: 'abc123321cba'
    swagger_file: my_api.yml
    cache_enabled: true
    cache_size: '6.1'
    canary_settings:
      percentTraffic: 50.0
      deploymentId: '123'
      useStageCache: true
    state: present

- name: Delete API gateway
  amazon.aws.api_gateway:
    name: ansible-rest-api
    tags:
      automation: ansible
    lookup: tags
    state: absent
"""

RETURN = r"""
api_id:
    description: API id of the API endpoint created
    returned: success
    type: str
    sample: '0ln4zq7p86'
configure_response:
    description: AWS response from the API configure call
    returned: success
    type: dict
    sample: { api_key_source: "HEADER", created_at: "2020-01-01T11:37:59+00:00", id: "0ln4zq7p86" }
deploy_response:
    description: AWS response from the API deploy call
    returned: success
    type: dict
    sample: { created_date: "2020-01-01T11:36:59+00:00", id: "rptv4b", description: "Automatic deployment by Ansible." }
resource_actions:
    description: Actions performed against AWS API
    returned: always
    type: list
    sample: ["apigateway:CreateRestApi", "apigateway:CreateDeployment", "apigateway:PutRestApi"]
"""

import json
import traceback

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def main():
    argument_spec = dict(
        api_id=dict(type="str", required=False),
        state=dict(type="str", default="present", choices=["present", "absent"]),
        swagger_file=dict(type="path", default=None, aliases=["src", "api_file"]),
        swagger_dict=dict(type="json", default=None),
        swagger_text=dict(type="str", default=None),
        stage=dict(type="str", default=None),
        deploy_desc=dict(type="str", default="Automatic deployment by Ansible."),
        cache_enabled=dict(type="bool", default=False),
        cache_size=dict(type="str", default="0.5", choices=["0.5", "1.6", "6.1", "13.5", "28.4", "58.2", "118", "237"]),
        stage_variables=dict(type="dict", default={}),
        stage_canary_settings=dict(type="dict", default={}),
        tracing_enabled=dict(type="bool", default=False),
        endpoint_type=dict(type="str", default="EDGE", choices=["EDGE", "REGIONAL", "PRIVATE"]),
        name=dict(type="str"),
        lookup=dict(type="str", choices=["tag", "id"], default="tag"),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(default=True, type="bool"),
    )

    mutually_exclusive = [["swagger_file", "swagger_dict", "swagger_text"]]  # noqa: F841

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
    )

    api_id = module.params.get("api_id")
    state = module.params.get("state")  # noqa: F841
    swagger_file = module.params.get("swagger_file")
    swagger_dict = module.params.get("swagger_dict")
    swagger_text = module.params.get("swagger_text")
    endpoint_type = module.params.get("endpoint_type")
    name = module.params.get("name")
    tags = module.params.get("tags")
    lookup = module.params.get("lookup")

    client = module.client("apigateway")

    changed = True  # for now it will stay that way until we can sometimes avoid change
    conf_res = None
    dep_res = None
    del_res = None

    if state == "present":
        if api_id is None:
            # lookup API gateway using tags
            if tags and lookup == "tag":
                rest_api = get_api_by_tags(client, module, name, tags)
                if rest_api:
                    api_id = rest_api["id"]
        if module.check_mode:
            module.exit_json(changed=True, msg="Create/update operation skipped - running in check mode.")
        if api_id is None:
            api_data = get_api_definitions(
                module, swagger_file=swagger_file, swagger_dict=swagger_dict, swagger_text=swagger_text
            )
            # create new API gateway as non were provided and/or found using lookup=tag
            api_id = create_empty_api(module, client, name, endpoint_type, tags)
            conf_res, dep_res = ensure_api_in_correct_state(module, client, api_id, api_data)
        tags = module.params.get("tags")
        purge_tags = module.params.get("purge_tags")
        if tags:
            if not conf_res:
                conf_res = get_rest_api(module, client, api_id=api_id)
            tag_changed, tag_result = ensure_apigateway_tags(
                module, client, api_id=api_id, current_tags=conf_res.get("tags"), new_tags=tags, purge_tags=purge_tags
            )
            if tag_changed:
                changed |= tag_changed
                conf_res = tag_result
    if state == "absent":
        if api_id is None:
            if lookup != "tag" or not tags:
                module.fail_json(
                    msg="API gateway id must be supplied to delete API gateway or provided tag with lookup=tag to identify API gateway id."
                )
            rest_api = get_api_by_tags(client, module, name, tags)
            if not rest_api:
                module.exit_json(changed=False, msg="No API gateway identified with tags provided")
            api_id = rest_api["id"]
        elif not describe_api(client, module, api_id):
            module.exit_json(changed=False, msg=f"API gateway id '{api_id}' does not exist.")

        if module.check_mode:
            module.exit_json(changed=True, msg="Delete operation skipped - running in check mode.", api_id=api_id)

        del_res = delete_rest_api(module, client, api_id)

    exit_args = {"changed": changed, "api_id": api_id}

    if conf_res is not None:
        exit_args["configure_response"] = camel_dict_to_snake_dict(conf_res)
    if dep_res is not None:
        exit_args["deploy_response"] = camel_dict_to_snake_dict(dep_res)
    if del_res is not None:
        exit_args["delete_response"] = camel_dict_to_snake_dict(del_res)

    module.exit_json(**exit_args)


def ensure_apigateway_tags(module, client, api_id, current_tags, new_tags, purge_tags):
    changed = False
    tag_result = {}
    tags_to_set, tags_to_delete = compare_aws_tags(current_tags, new_tags, purge_tags)
    if tags_to_set or tags_to_delete:
        changed = True
        apigateway_arn = f"arn:aws:apigateway:{module.region}::/restapis/{api_id}"
        # Remove tags from Resource
        if tags_to_delete:
            client.untag_resource(resourceArn=apigateway_arn, tagKeys=tags_to_delete)
        # add new tags to resource
        if tags_to_set:
            client.tag_resource(resourceArn=apigateway_arn, tags=tags_to_set)
        # Describe API gateway
        tag_result = get_rest_api(module, client, api_id=api_id)
    return changed, tag_result


def get_api_definitions(module, swagger_file=None, swagger_dict=None, swagger_text=None):
    apidata = None
    if swagger_file is not None:
        try:
            with open(swagger_file) as f:
                apidata = f.read()
        except OSError as e:
            msg = f"Failed trying to read swagger file {str(swagger_file)}: {str(e)}"
            module.fail_json(msg=msg, exception=traceback.format_exc())
    if swagger_dict is not None:
        apidata = json.dumps(swagger_dict)
    if swagger_text is not None:
        apidata = swagger_text

    if apidata is None:
        module.fail_json(msg="module error - no swagger info provided")
    return apidata


def get_rest_api(module, client, api_id):
    try:
        response = client.get_rest_api(restApiId=api_id)
        response.pop("ResponseMetadata", None)
        return response
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg=f"failed to get REST API with api_id={api_id}")


def create_empty_api(module, client, name, endpoint_type, tags):
    """
    creates a new empty API ready to be configured. The description is
    temporarily set to show the API as incomplete but should be
    updated when the API is configured.
    """
    desc = "Incomplete API creation by ansible api_gateway module"
    try:
        rest_api_name = name or "ansible-temp-api"
        awsret = create_api(client, name=rest_api_name, description=desc, endpoint_type=endpoint_type, tags=tags)
    except (botocore.exceptions.ClientError, botocore.exceptions.EndpointConnectionError) as e:
        module.fail_json_aws(e, msg="creating API")
    return awsret["id"]


def delete_rest_api(module, client, api_id):
    """
    Deletes entire REST API setup
    """
    try:
        delete_response = delete_api(client, api_id)
    except (botocore.exceptions.ClientError, botocore.exceptions.EndpointConnectionError) as e:
        module.fail_json_aws(e, msg=f"deleting API {api_id}")
    return delete_response


def ensure_api_in_correct_state(module, client, api_id, api_data):
    """Make sure that we have the API configured and deployed as instructed.

    This function first configures the API correctly uploading the
    swagger definitions and then deploys those.  Configuration and
    deployment should be closely tied because there is only one set of
    definitions so if we stop, they may be updated by someone else and
    then we deploy the wrong configuration.
    """

    configure_response = None
    try:
        configure_response = configure_api(client, api_id, api_data=api_data)
        configure_response.pop("ResponseMetadata", None)
    except (botocore.exceptions.ClientError, botocore.exceptions.EndpointConnectionError) as e:
        module.fail_json_aws(e, msg=f"configuring API {api_id}")

    deploy_response = None

    stage = module.params.get("stage")
    if stage:
        try:
            deploy_response = create_deployment(client, api_id, **module.params)
            deploy_response.pop("ResponseMetadata", None)
        except (botocore.exceptions.ClientError, botocore.exceptions.EndpointConnectionError) as e:
            msg = f"deploying api {api_id} to stage {stage}"
            module.fail_json_aws(e, msg)

    return configure_response, deploy_response


def get_api_by_tags(client, module, name, tags):
    count = 0
    result = None
    for api in list_apis(client):
        if name and api["name"] != name:
            continue
        api_tags = api.get("tags", {})
        if all((tag_key in api_tags and api_tags[tag_key] == tag_value for tag_key, tag_value in tags.items())):
            result = api
            count += 1

    if count > 1:
        args = "Tags"
        if name:
            args += " and name"
        module.fail_json(msg=f"{args} provided do not identify a unique API gateway")
    return result


retry_params = {"retries": 10, "delay": 10, "catch_extra_error_codes": ["TooManyRequestsException"]}


@AWSRetry.jittered_backoff(**retry_params)
def create_api(client, name, description=None, endpoint_type=None, tags=None):
    params = {"name": name}
    if description:
        params["description"] = description
    if endpoint_type:
        params["endpointConfiguration"] = {"types": [endpoint_type]}
    if tags:
        params["tags"] = tags
    return client.create_rest_api(**params)


@AWSRetry.jittered_backoff(**retry_params)
def delete_api(client, api_id):
    return client.delete_rest_api(restApiId=api_id)


@AWSRetry.jittered_backoff(**retry_params)
def configure_api(client, api_id, api_data=None, mode="overwrite"):
    return client.put_rest_api(restApiId=api_id, mode=mode, body=api_data)


@AWSRetry.jittered_backoff(**retry_params)
def create_deployment(client, rest_api_id, **params):
    canary_settings = params.get("stage_canary_settings")

    if canary_settings and len(canary_settings) > 0:
        result = client.create_deployment(
            restApiId=rest_api_id,
            stageName=params.get("stage"),
            description=params.get("deploy_desc"),
            cacheClusterEnabled=params.get("cache_enabled"),
            cacheClusterSize=params.get("cache_size"),
            variables=params.get("stage_variables"),
            canarySettings=canary_settings,
            tracingEnabled=params.get("tracing_enabled"),
        )
    else:
        result = client.create_deployment(
            restApiId=rest_api_id,
            stageName=params.get("stage"),
            description=params.get("deploy_desc"),
            cacheClusterEnabled=params.get("cache_enabled"),
            cacheClusterSize=params.get("cache_size"),
            variables=params.get("stage_variables"),
            tracingEnabled=params.get("tracing_enabled"),
        )

    return result


@AWSRetry.jittered_backoff(**retry_params)
def list_apis(client):
    paginator = client.get_paginator("get_rest_apis")
    return paginator.paginate().build_full_result().get("items", [])


@AWSRetry.jittered_backoff(**retry_params)
def describe_api(client, module, rest_api_id):
    try:
        response = client.get_rest_api(restApiId=rest_api_id)
        response.pop("ResponseMetadata")
    except is_boto3_error_code("ResourceNotFoundException"):
        response = {}
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg=f"Trying to get Rest API '{rest_api_id}'.")
    return response


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Piotr Wojciechowski <piotr@it-playground.pl>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
---
module: docker_node
short_description: Manage Docker Swarm node
description:
    - Manages the Docker nodes via Swarm Manager.
    - This module allows to change the node's role, its availability, and to modify, add or remove node labels.

extends_documentation_fragment:
  - community.docker.docker
  - community.docker.docker.docker_py_1_documentation
  - community.docker.attributes
  - community.docker.attributes.actiongroup_docker

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
    hostname:
        description:
            - The hostname or ID of node as registered in Swarm.
            - If more than one node is registered using the same hostname the ID must be used,
              otherwise module will fail.
        type: str
        required: true
    labels:
        description:
            - User-defined key/value metadata that will be assigned as node attribute.
            - Label operations in this module apply to the docker swarm node specified by O(hostname).
              Use M(community.docker.docker_swarm) module to add/modify/remove swarm cluster labels.
            - The actual state of labels assigned to the node when module completes its work depends on
              O(labels_state) and O(labels_to_remove) parameters values. See description below.
        type: dict
    labels_state:
        description:
            - It defines the operation on the labels assigned to node and labels specified in O(labels) option.
            - Set to V(merge) to combine labels provided in O(labels) with those already assigned to the node.
              If no labels are assigned then it will add listed labels. For labels that are already assigned
              to the node, it will update their values. The labels not specified in O(labels) will remain unchanged.
              If O(labels) is empty then no changes will be made.
            - Set to V(replace) to replace all assigned labels with provided ones. If O(labels) is empty then
              all labels assigned to the node will be removed.
        type: str
        default: 'merge'
        choices:
          - merge
          - replace
    labels_to_remove:
        description:
            - List of labels that will be removed from the node configuration. The list has to contain only label
              names, not their values.
            - If the label provided on the list is not assigned to the node, the entry is ignored.
            - If the label is both on the O(labels_to_remove) and O(labels), then value provided in O(labels) remains
              assigned to the node.
            - If O(labels_state=replace) and O(labels) is not provided or empty then all labels assigned to
              node are removed and O(labels_to_remove) is ignored.
        type: list
        elements: str
    availability:
        description: Node availability to assign. If not provided then node availability remains unchanged.
        choices:
          - active
          - pause
          - drain
        type: str
    role:
        description: Node role to assign. If not provided then node role remains unchanged.
        choices:
          - manager
          - worker
        type: str

requirements:
  - "L(Docker SDK for Python,https://docker-py.readthedocs.io/en/stable/) >= 2.4.0"
  - Docker API >= 1.25
author:
  - Piotr Wojciechowski (@WojciechowskiPiotr)
  - Thierry Bouvet (@tbouvet)

'''

EXAMPLES = '''
- name: Set node role
  community.docker.docker_node:
    hostname: mynode
    role: manager

- name: Set node availability
  community.docker.docker_node:
    hostname: mynode
    availability: drain

- name: Replace node labels with new labels
  community.docker.docker_node:
    hostname: mynode
    labels:
      key: value
    labels_state: replace

- name: Merge node labels and new labels
  community.docker.docker_node:
    hostname: mynode
    labels:
      key: value

- name: Remove all labels assigned to node
  community.docker.docker_node:
    hostname: mynode
    labels_state: replace

- name: Remove selected labels from the node
  community.docker.docker_node:
    hostname: mynode
    labels_to_remove:
      - key1
      - key2
'''

RETURN = '''
node:
  description: Information about node after 'update' operation
  returned: success
  type: dict

'''

import traceback

try:
    from docker.errors import DockerException, APIError
except ImportError:
    # missing Docker SDK for Python handled in ansible.module_utils.docker.common
    pass

from ansible_collections.community.docker.plugins.module_utils.common import (
    DockerBaseClass,
    RequestException,
)

from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.docker.plugins.module_utils.swarm import AnsibleDockerSwarmClient


class TaskParameters(DockerBaseClass):
    def __init__(self, client):
        super(TaskParameters, self).__init__()

        # Spec
        self.name = None
        self.labels = None
        self.labels_state = None
        self.labels_to_remove = None

        # Node
        self.availability = None
        self.role = None

        for key, value in client.module.params.items():
            setattr(self, key, value)


class SwarmNodeManager(DockerBaseClass):

    def __init__(self, client, results):

        super(SwarmNodeManager, self).__init__()

        self.client = client
        self.results = results
        self.check_mode = self.client.check_mode

        self.client.fail_task_if_not_swarm_manager()

        self.parameters = TaskParameters(client)

        self.node_update()

    def node_update(self):
        if not (self.client.check_if_swarm_node(node_id=self.parameters.hostname)):
            self.client.fail("This node is not part of a swarm.")
            return

        if self.client.check_if_swarm_node_is_down():
            self.client.fail("Can not update the node. The node is down.")

        try:
            node_info = self.client.inspect_node(node_id=self.parameters.hostname)
        except APIError as exc:
            self.client.fail("Failed to get node information for %s" % to_native(exc))

        changed = False
        node_spec = dict(
            Availability=self.parameters.availability,
            Role=self.parameters.role,
            Labels=self.parameters.labels,
        )

        if self.parameters.role is None:
            node_spec['Role'] = node_info['Spec']['Role']
        else:
            if not node_info['Spec']['Role'] == self.parameters.role:
                node_spec['Role'] = self.parameters.role
                changed = True

        if self.parameters.availability is None:
            node_spec['Availability'] = node_info['Spec']['Availability']
        else:
            if not node_info['Spec']['Availability'] == self.parameters.availability:
                node_info['Spec']['Availability'] = self.parameters.availability
                changed = True

        if self.parameters.labels_state == 'replace':
            if self.parameters.labels is None:
                node_spec['Labels'] = {}
                if node_info['Spec']['Labels']:
                    changed = True
            else:
                if (node_info['Spec']['Labels'] or {}) != self.parameters.labels:
                    node_spec['Labels'] = self.parameters.labels
                    changed = True
        elif self.parameters.labels_state == 'merge':
            node_spec['Labels'] = dict(node_info['Spec']['Labels'] or {})
            if self.parameters.labels is not None:
                for key, value in self.parameters.labels.items():
                    if node_spec['Labels'].get(key) != value:
                        node_spec['Labels'][key] = value
                        changed = True

            if self.parameters.labels_to_remove is not None:
                for key in self.parameters.labels_to_remove:
                    if self.parameters.labels is not None:
                        if not self.parameters.labels.get(key):
                            if node_spec['Labels'].get(key):
                                node_spec['Labels'].pop(key)
                                changed = True
                        else:
                            self.client.module.warn(
                                "Label '%s' listed both in 'labels' and 'labels_to_remove'. "
                                "Keeping the assigned label value."
                                % to_native(key))
                    else:
                        if node_spec['Labels'].get(key):
                            node_spec['Labels'].pop(key)
                            changed = True

        if changed is True:
            if not self.check_mode:
                try:
                    self.client.update_node(node_id=node_info['ID'], version=node_info['Version']['Index'],
                                            node_spec=node_spec)
                except APIError as exc:
                    self.client.fail("Failed to update node : %s" % to_native(exc))
            self.results['node'] = self.client.get_node_inspect(node_id=node_info['ID'])
            self.results['changed'] = changed
        else:
            self.results['node'] = node_info
            self.results['changed'] = changed


def main():
    argument_spec = dict(
        hostname=dict(type='str', required=True),
        labels=dict(type='dict'),
        labels_state=dict(type='str', default='merge', choices=['merge', 'replace']),
        labels_to_remove=dict(type='list', elements='str'),
        availability=dict(type='str', choices=['active', 'pause', 'drain']),
        role=dict(type='str', choices=['worker', 'manager']),
    )

    client = AnsibleDockerSwarmClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
        min_docker_version='2.4.0',
    )

    try:
        results = dict(
            changed=False,
        )

        SwarmNodeManager(client, results)
        client.module.exit_json(**results)
    except DockerException as e:
        client.fail('An unexpected docker error occurred: {0}'.format(to_native(e)), exception=traceback.format_exc())
    except RequestException as e:
        client.fail(
            'An unexpected requests error occurred when Docker SDK for Python tried to talk to the docker daemon: {0}'.format(to_native(e)),
            exception=traceback.format_exc())


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_l3out_extepg
short_description: Manage External Network Instance Profile (ExtEpg) objects (l3extInstP:instP)
description:
- Manage External Network Instance Profile (ExtEpg) objects (l3extInstP:instP)
options:
  tenant:
    description:
    - Name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  l3out:
    description:
    - Name of an existing L3Out.
    type: str
    aliases: [ l3out_name ]
  extepg:
    description:
    - Name of ExtEpg being created.
    type: str
    aliases: [ extepg_name, name ]
  description:
    description:
    - Description for the ExtEpg.
    type: str
    aliases: [ descr ]
  preferred_group:
    description:
    - Whether ot not the EPG is part of the Preferred Group and can communicate without contracts.
    - This is very convenient for migration scenarios, or when ACI is used for network automation but not for policy.
    - The APIC defaults to C(false) when unset during creation.
    type: bool
  dscp:
    description:
    - The target Differentiated Service (DSCP) value.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ AF11, AF12, AF13, AF21, AF22, AF23, AF31, AF32, AF33, AF41, AF42, AF43, CS0, CS1, CS2, CS3, CS4, CS5, CS6, CS7, EF, VA, unspecified ]
    aliases: [ target ]
  contract_exception_tag:
    description:
    - The tag for contract exception.
    type: str
  qos_class:
    description:
    - The priority class identifier.
    type: str
    choices: [ level1, level2, level3, level4, level5, level6, unspecified ]
    default: level3
  intra_ext_epg_isolation:
    description:
    - The preferred policy control.
    type: str
    choices: [ enforced, unenforced ]
  route_control_profiles:
    description:
    - The route control profiles.
    type: dict
    suboptions:
      import_profile:
        description:
        - The route control profile whose direction is import.
        - To remove a route import policy pass an empty string (see Examples).
        type: str
        aliases: [ import ]
      export_profile:
        description:
         - The route control profile whose direction is export.
         - To remove a route export policy pass an empty string (see Examples).
        type: str
        aliases: [ export ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The C(tenant) and C(domain) and C(vrf) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) and M(cisco.aci.aci_domain) and M(cisco.aci.aci_vrf) modules can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_domain
- module: cisco.aci.aci_vrf
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(l3ext:Out).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Rostyslav Davydenko (@rost-d)
"""

EXAMPLES = r"""
- name: Add a new ExtEpg
  cisco.aci.aci_l3out_extepg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    l3out: prod_l3out
    name: prod_extepg
    description: ExtEpg for Production L3Out
    contract_exception_tag: test
    qos_class: level6
    route_control_profiles:
      import_profile: test1
      export_profile: test2
    state: present
  delegate_to: localhost

- name: Delete ExtEpg
  cisco.aci.aci_l3out_extepg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    l3out: prod_l3out
    name: prod_extepg
    state: absent
  delegate_to: localhost

- name: Delete the export route control profile in an ExtEpg
  cisco.aci.aci_l3out_extepg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    l3out: prod_l3out
    name: prod_extepg
    route_control_profiles:
      import_profile: test1
      export_profile: ""
    state: present
  delegate_to: localhost

- name: Query ExtEpg information
  cisco.aci.aci_l3out_extepg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    l3out: prod_l3out
    name: prod_extepg
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        l3out=dict(type="str", aliases=["l3out_name"]),  # Not required for querying all objects
        extepg=dict(type="str", aliases=["extepg_name", "name"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        preferred_group=dict(type="bool"),
        dscp=dict(
            type="str",
            choices=[
                "AF11",
                "AF12",
                "AF13",
                "AF21",
                "AF22",
                "AF23",
                "AF31",
                "AF32",
                "AF33",
                "AF41",
                "AF42",
                "AF43",
                "CS0",
                "CS1",
                "CS2",
                "CS3",
                "CS4",
                "CS5",
                "CS6",
                "CS7",
                "EF",
                "VA",
                "unspecified",
            ],
            aliases=["target"],
        ),
        contract_exception_tag=dict(type="str"),
        qos_class=dict(type="str", default="level3", choices=["level1", "level2", "level3", "level4", "level5", "level6", "unspecified"]),
        intra_ext_epg_isolation=dict(type="str", choices=["enforced", "unenforced"]),
        route_control_profiles=dict(
            type="dict",
            options=dict(import_profile=dict(type="str", aliases=["import"]), export_profile=dict(type="str", aliases=["export"])),
        ),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["extepg", "l3out", "tenant"]],
            ["state", "absent", ["extepg", "l3out", "tenant"]],
        ],
    )

    aci = ACIModule(module)

    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    extepg = module.params.get("extepg")
    description = module.params.get("description")
    preferred_group = aci.boolean(module.params.get("preferred_group"), "include", "exclude")
    dscp = module.params.get("dscp")
    state = module.params.get("state")
    contract_exception_tag = module.params.get("contract_exception_tag")
    qos_class = module.params.get("qos_class")
    intra_ext_epg_isolation = module.params.get("intra_ext_epg_isolation")
    route_control_profiles = module.params.get("route_control_profiles")
    name_alias = module.params.get("name_alias")

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extInstP",
            aci_rn="instP-{0}".format(extepg),
            module_object=extepg,
            target_filter={"name": extepg},
        ),
        child_classes=["fvRsSecInherited", "l3extRsInstPToProfile"],
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        if route_control_profiles:
            for key, name in route_control_profiles.items():
                if "_profile" in key:
                    direction = key.rstrip("_profile")
                    if name == "" and isinstance(aci.existing, list) and len(aci.existing) > 0:
                        for child in aci.existing[0].get("l3extInstP", {}).get("children", {}):
                            if child.get("l3extRsInstPToProfile") and child.get("l3extRsInstPToProfile").get("attributes").get("direction") == direction:
                                child_configs.append(
                                    dict(
                                        l3extRsInstPToProfile=dict(
                                            attributes=dict(
                                                status="deleted",
                                                direction=direction,
                                                tnRtctrlProfileName=child.get("l3extRsInstPToProfile").get("attributes").get("tnRtctrlProfileName"),
                                            )
                                        )
                                    )
                                )
                    elif name:
                        child_configs.append(dict(l3extRsInstPToProfile=dict(attributes=dict(direction=direction, tnRtctrlProfileName=name))))

        class_config = dict(
            name=extepg,
            descr=description,
            prefGrMemb=preferred_group,
            targetDscp=dscp,
            nameAlias=name_alias,
            prio=qos_class,
            exceptionTag=contract_exception_tag,
        )

        if intra_ext_epg_isolation:
            class_config.update(pcEnfPref=intra_ext_epg_isolation)

        aci.payload(aci_class="l3extInstP", class_config=class_config, child_configs=child_configs)

        aci.get_diff(aci_class="l3extInstP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import io
import os
import json
import datetime
import importlib
from contextlib import redirect_stdout, suppress
from unittest import mock
import logging

from requests.models import Response, PreparedRequest

import pytest

from ansible.module_utils.six import raise_from

from awx.main.tests.functional.conftest import _request
from awx.main.tests.functional.conftest import credentialtype_scm, credentialtype_ssh  # noqa: F401; pylint: disable=unused-variable
from awx.main.models import (
    Organization,
    Project,
    Inventory,
    JobTemplate,
    Credential,
    CredentialType,
    ExecutionEnvironment,
    UnifiedJob,
    WorkflowJobTemplate,
    NotificationTemplate,
    Schedule,
)

from django.db import transaction


HAS_TOWER_CLI = False
HAS_AWX_KIT = False
logger = logging.getLogger('awx.main.tests')


@pytest.fixture(autouse=True)
def awxkit_path_set(monkeypatch):
    """Monkey patch sys.path, insert awxkit source code so that
    the package does not need to be installed.
    """
    base_folder = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir, os.pardir, os.pardir, 'awxkit'))
    monkeypatch.syspath_prepend(base_folder)


@pytest.fixture(autouse=True)
def import_awxkit():
    global HAS_TOWER_CLI
    global HAS_AWX_KIT
    try:
        import tower_cli  # noqa
        HAS_TOWER_CLI = True
    except ImportError:
        HAS_TOWER_CLI = False

    try:
        import awxkit  # noqa
        HAS_AWX_KIT = True
    except ImportError:
        HAS_AWX_KIT = False


def sanitize_dict(din):
    """Sanitize Django response data to purge it of internal types
    so it may be used to cast a requests response object
    """
    if isinstance(din, (int, str, type(None), bool)):
        return din  # native JSON types, no problem
    elif isinstance(din, datetime.datetime):
        return din.isoformat()
    elif isinstance(din, list):
        for i in range(len(din)):
            din[i] = sanitize_dict(din[i])
        return din
    elif isinstance(din, dict):
        for k in din.copy().keys():
            din[k] = sanitize_dict(din[k])
        return din
    else:
        return str(din)  # translation proxies often not string but stringlike


@pytest.fixture(autouse=True)
def collection_path_set(monkeypatch):
    """Monkey patch sys.path, insert the root of the collection folder
    so that content can be imported without being fully packaged
    """
    base_folder = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir, os.pardir))
    monkeypatch.syspath_prepend(base_folder)


@pytest.fixture
def collection_import():
    """These tests run assuming that the awx_collection folder is inserted
    into the PATH before-hand by collection_path_set.
    But all imports internally to the collection
    go through this fixture so that can be changed if needed.
    For instance, we could switch to fully-qualified import paths.
    """

    def rf(path):
        return importlib.import_module(path)

    return rf


@pytest.fixture
def run_module(request, collection_import):
    def rf(module_name, module_params, request_user):
        def new_request(self, method, url, **kwargs):
            kwargs_copy = kwargs.copy()
            if 'data' in kwargs:
                if isinstance(kwargs['data'], dict):
                    kwargs_copy['data'] = kwargs['data']
                elif kwargs['data'] is None:
                    pass
                elif isinstance(kwargs['data'], str):
                    kwargs_copy['data'] = json.loads(kwargs['data'])
                else:
                    raise RuntimeError('Expected data to be dict or str, got {0}, data: {1}'.format(type(kwargs['data']), kwargs['data']))
            if 'params' in kwargs and method == 'GET':
                # query params for GET are handled a bit differently by
                # tower-cli and python requests as opposed to REST framework APIRequestFactory
                if not kwargs_copy.get('data'):
                    kwargs_copy['data'] = {}
                if isinstance(kwargs['params'], dict):
                    kwargs_copy['data'].update(kwargs['params'])
                elif isinstance(kwargs['params'], list):
                    for k, v in kwargs['params']:
                        kwargs_copy['data'][k] = v

            # make request
            with transaction.atomic():
                rf = _request(method.lower())
                django_response = rf(url, user=request_user, expect=None, **kwargs_copy)

            # requests library response object is different from the Django response, but they are the same concept
            # this converts the Django response object into a requests response object for consumption
            resp = Response()
            py_data = django_response.data
            sanitize_dict(py_data)
            resp._content = bytes(json.dumps(django_response.data), encoding='utf8')
            resp.status_code = django_response.status_code
            resp.headers = dict(django_response.headers)

            if request.config.getoption('verbose') > 0:
                logger.info('%s %s by %s, code:%s', method, '/api/' + url.split('/api/')[1], request_user.username, resp.status_code)

            resp.request = PreparedRequest()
            resp.request.prepare(method=method, url=url)
            return resp

        def new_open(self, method, url, **kwargs):
            r = new_request(self, method, url, **kwargs)
            m = mock.MagicMock(read=mock.MagicMock(return_value=r._content), status=r.status_code, getheader=mock.MagicMock(side_effect=r.headers.get))
            return m

        stdout_buffer = io.StringIO()
        # Requies specific PYTHONPATH, see docs
        # Note that a proper Ansiballz explosion of the modules will have an import path like:
        # ansible_collections.awx.awx.plugins.modules.{}
        # We should consider supporting that in the future
        resource_module = collection_import('plugins.modules.{0}'.format(module_name))

        if not isinstance(module_params, dict):
            raise RuntimeError('Module params must be dict, got {0}'.format(type(module_params)))

        # Ansible params can be passed as an invocation argument or over stdin
        # this short circuits within the AnsibleModule interface
        def mock_load_params(self):
            self.params = module_params

        if getattr(resource_module, 'ControllerAWXKitModule', None):
            resource_class = resource_module.ControllerAWXKitModule
        elif getattr(resource_module, 'ControllerAPIModule', None):
            resource_class = resource_module.ControllerAPIModule
        else:
            raise RuntimeError("The module has neither a ControllerAWXKitModule or a ControllerAPIModule")

        with mock.patch.object(resource_class, '_load_params', new=mock_load_params):
            # Call the test utility (like a mock server) instead of issuing HTTP requests
            with mock.patch('ansible.module_utils.urls.Request.open', new=new_open):
                if HAS_TOWER_CLI:
                    tower_cli_mgr = mock.patch('tower_cli.api.Session.request', new=new_request)
                elif HAS_AWX_KIT:
                    tower_cli_mgr = mock.patch('awxkit.api.client.requests.Session.request', new=new_request)
                else:
                    tower_cli_mgr = suppress()
                with tower_cli_mgr:
                    try:
                        # Ansible modules return data to the mothership over stdout
                        with redirect_stdout(stdout_buffer):
                            resource_module.main()
                    except SystemExit:
                        pass  # A system exit indicates successful execution
                    except Exception:
                        # dump the stdout back to console for debugging
                        print(stdout_buffer.getvalue())
                        raise

        module_stdout = stdout_buffer.getvalue().strip()
        try:
            result = json.loads(module_stdout)
        except Exception as e:
            raise_from(Exception('Module did not write valid JSON, error: {0}, stdout:\n{1}'.format(str(e), module_stdout)), e)
        # A module exception should never be a test expectation
        if 'exception' in result:
            if "ModuleNotFoundError: No module named 'tower_cli'" in result['exception']:
                pytest.skip('The tower-cli library is needed to run this test, module no longer supported.')
            raise Exception('Module encountered error:\n{0}'.format(result['exception']))
        return result

    return rf


@pytest.fixture
def survey_spec():
    return {
        "spec": [{"index": 0, "question_name": "my question?", "default": "mydef", "variable": "myvar", "type": "text", "required": False}],
        "description": "test",
        "name": "test",
    }


@pytest.fixture
def organization():
    return Organization.objects.create(name='Default')


@pytest.fixture
def project(organization):
    return Project.objects.create(
        name="test-proj",
        description="test-proj-desc",
        organization=organization,
        playbook_files=['helloworld.yml'],
        local_path='_92__test_proj',
        scm_revision='1234567890123456789012345678901234567890',
        scm_url='localhost',
        scm_type='git',
    )


@pytest.fixture
def inventory(organization):
    return Inventory.objects.create(name='test-inv', organization=organization)


@pytest.fixture
def job_template(project, inventory):
    return JobTemplate.objects.create(name='test-jt', project=project, inventory=inventory, playbook='helloworld.yml')


@pytest.fixture
def machine_credential(credentialtype_ssh, organization):  # noqa: F811
    return Credential.objects.create(credential_type=credentialtype_ssh, name='machine-cred', inputs={'username': 'test_user', 'password': 'pas4word'})


@pytest.fixture
def vault_credential(organization):
    ct = CredentialType.defaults['vault']()
    ct.save()
    return Credential.objects.create(credential_type=ct, name='vault-cred', inputs={'vault_id': 'foo', 'vault_password': 'pas4word'})


@pytest.fixture
def kube_credential():
    ct = CredentialType.defaults['kubernetes_bearer_token']()
    ct.save()
    return Credential.objects.create(credential_type=ct, name='kube-cred', inputs={'host': 'my.cluster', 'bearer_token': 'my-token', 'verify_ssl': False})


@pytest.fixture
def silence_deprecation():
    """The deprecation warnings are stored in a global variable
    they will create cross-test interference. Use this to turn them off.
    """
    with mock.patch('ansible.module_utils.basic.AnsibleModule.deprecate') as this_mock:
        yield this_mock


@pytest.fixture(autouse=True)
def silence_warning():
    """Warnings use global variable, same as deprecations."""
    with mock.patch('ansible.module_utils.basic.AnsibleModule.warn') as this_mock:
        yield this_mock


@pytest.fixture
def execution_environment():
    return ExecutionEnvironment.objects.create(name="test-ee", description="test-ee", managed=False)


@pytest.fixture(scope='session', autouse=True)
def mock_has_unpartitioned_events():
    # has_unpartitioned_events determines if there are any events still
    # left in the old, unpartitioned job events table. In order to work,
    # this method looks up when the partition migration occurred. When
    # Django's unit tests run, however, there will be no record of the migration.
    # We mock this out to circumvent the migration query.
    with mock.patch.object(UnifiedJob, 'has_unpartitioned_events', new=False) as _fixture:
        yield _fixture


@pytest.fixture
def workflow_job_template(organization, inventory):
    return WorkflowJobTemplate.objects.create(name='test-workflow_job_template', organization=organization, inventory=inventory)


@pytest.fixture
def notification_template(organization):
    return NotificationTemplate.objects.create(
        name='test-notification_template',
        organization=organization,
        notification_type="webhook",
        notification_configuration=dict(
            url="http://localhost",
            username="",
            password="",
            headers={
                "Test": "Header",
            },
        ),
    )


@pytest.fixture
def scm_credential(credentialtype_scm, organization):  # noqa: F811
    return Credential.objects.create(
        credential_type=credentialtype_scm, name='scm-cred', inputs={'username': 'optimus', 'password': 'prime'}, organization=organization
    )


@pytest.fixture
def rrule():
    return 'DTSTART:20151117T050000Z RRULE:FREQ=DAILY;INTERVAL=1;COUNT=1'


@pytest.fixture
def schedule(job_template, rrule):
    return Schedule.objects.create(unified_job_template=job_template, name='test-sched', rrule=rrule)



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_snmp_contact
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: (deprecated, removed after 2024-01-01) Manages SNMP contact info.
description:
- Manages SNMP contact information.
version_added: 1.0.0
deprecated:
  alternative: nxos_snmp_server
  why: Updated modules released with more functionality
  removed_at_date: '2024-01-01'
author:
- Jason Edelman (@jedelman8)
- Gabriele Gerbino (@GGabriele)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Limited Support for Cisco MDS
- C(state=absent) removes the contact configuration if it is configured.
options:
  contact:
    description:
    - Contact information.
    required: true
    type: str
  state:
    description:
    - Manage the state of the resource.
    default: present
    choices:
    - present
    - absent
    type: str
"""

EXAMPLES = """
# ensure snmp contact is configured
- cisco.nxos.nxos_snmp_contact:
    contact: Test
    state: present
"""

RETURN = """
commands:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["snmp-server contact New_Test"]
"""


import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    load_config,
    run_commands,
)


def execute_show_command(command, module):
    command = {"command": command, "output": "text"}

    return run_commands(module, command)


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def get_snmp_contact(module):
    contact = {}
    contact_regex = r"^\s*snmp-server\scontact\s(?P<contact>.+)$"

    body = execute_show_command("show run snmp", module)[0]
    match_contact = re.search(contact_regex, body, re.M)
    if match_contact:
        contact["contact"] = match_contact.group("contact")

    return contact


def main():
    argument_spec = dict(
        contact=dict(required=True, type="str"),
        state=dict(choices=["absent", "present"], default="present"),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = list()
    results = {"changed": False, "commands": [], "warnings": warnings}

    contact = module.params["contact"]
    state = module.params["state"]

    existing = get_snmp_contact(module)
    commands = []

    if state == "absent":
        if existing and existing["contact"] == contact:
            commands.append("no snmp-server contact")
    elif state == "present":
        if not existing or existing["contact"] != contact:
            commands.append("snmp-server contact {0}".format(contact))

    cmds = flatten_list(commands)
    if cmds:
        results["changed"] = True
        if not module.check_mode:
            load_config(module, cmds)

        if "configure" in cmds:
            cmds.pop(0)
        results["commands"] = cmds

    module.exit_json(**results)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The cisco.ios bgp_address_family fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.bgp_address_family.bgp_address_family import (
    Bgp_address_familyArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.bgp_address_family import (
    Bgp_address_familyTemplate,
)


class Bgp_address_familyFacts(object):
    """The cisco.ios_bgp_address_family facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Bgp_address_familyArgs.argument_spec

    def get_bgp_address_family_data(self, connection):
        return connection.get("show running-config | section ^router bgp")

    def _process_facts(self, objs):
        """makes data as per the facts after data obtained from parsers"""
        addr_fam_facts = {}
        temp_af = []

        addr_fam_facts["as_number"] = objs["as_number"]
        if objs.get("address_family"):
            for kaf, afs in (objs["address_family"]).items():  # remove unique value from add_fam
                af = {}
                for af_key, afs_val in afs.items():
                    if af_key == "neighbors":
                        temp_neighbor = []
                        for tag, neighbor in afs_val.items():  # remove unique value from neighbor
                            if not neighbor.get("neighbor_address"):
                                neighbor["neighbor_address"] = tag
                            temp_neighbor.append(neighbor)
                        af[af_key] = temp_neighbor
                    else:
                        af[af_key] = afs_val
                temp_af.append(af)

        if temp_af:
            addr_fam_facts["address_family"] = temp_af
        return addr_fam_facts

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Bgp_address_family network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = {}

        if not data:
            data = self.get_bgp_address_family_data(connection)

        # parse native config using the Bgp_address_family template
        bgp_af_parser = Bgp_address_familyTemplate(lines=data.splitlines(), module=self._module)
        objs = bgp_af_parser.parse()
        if objs:
            objs = self._process_facts(utils.remove_empties(objs))
        ansible_facts["ansible_network_resources"].pop("bgp_address_family", None)
        params = utils.remove_empties(
            bgp_af_parser.validate_config(self.argument_spec, {"config": objs}, redact=True),
        )
        facts["bgp_address_family"] = params.get("config", {})
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



# -*- coding: utf-8 -*-

# Copyright (c) 2016, Hiroaki Nakamura <hnakamur@gmail.com>
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


import os
import socket
import ssl
import json

from ansible.module_utils.urls import generic_urlparse
from ansible.module_utils.six.moves.urllib.parse import urlparse
from ansible.module_utils.six.moves import http_client
from ansible.module_utils.common.text.converters import to_text

# httplib/http.client connection using unix domain socket
HTTPConnection = http_client.HTTPConnection
HTTPSConnection = http_client.HTTPSConnection


class UnixHTTPConnection(HTTPConnection):
    def __init__(self, path):
        HTTPConnection.__init__(self, 'localhost')
        self.path = path

    def connect(self):
        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        sock.connect(self.path)
        self.sock = sock


class LXDClientException(Exception):
    def __init__(self, msg, **kwargs):
        self.msg = msg
        self.kwargs = kwargs


class LXDClient(object):
    def __init__(self, url, key_file=None, cert_file=None, debug=False, server_cert_file=None, server_check_hostname=True):
        """LXD Client.

        :param url: The URL of the LXD server. (e.g. unix:/var/lib/lxd/unix.socket or https://127.0.0.1)
        :type url: ``str``
        :param key_file: The path of the client certificate key file.
        :type key_file: ``str``
        :param cert_file: The path of the client certificate file.
        :type cert_file: ``str``
        :param debug: The debug flag. The request and response are stored in logs when debug is true.
        :type debug: ``bool``
        :param server_cert_file: The path of the server certificate file.
        :type server_cert_file: ``str``
        :param server_check_hostname: Whether to check the server's hostname as part of TLS verification.
        :type debug: ``bool``
        """
        self.url = url
        self.debug = debug
        self.logs = []
        if url.startswith('https:'):
            self.cert_file = cert_file
            self.key_file = key_file
            parts = generic_urlparse(urlparse(self.url))
            ctx = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
            if server_cert_file:
                # Check that the received cert is signed by the provided server_cert_file
                ctx.load_verify_locations(cafile=server_cert_file)
            ctx.check_hostname = server_check_hostname
            ctx.load_cert_chain(cert_file, keyfile=key_file)
            self.connection = HTTPSConnection(parts.get('netloc'), context=ctx)
        elif url.startswith('unix:'):
            unix_socket_path = url[len('unix:'):]
            self.connection = UnixHTTPConnection(unix_socket_path)
        else:
            raise LXDClientException('URL scheme must be unix: or https:')

    def do(self, method, url, body_json=None, ok_error_codes=None, timeout=None, wait_for_container=None):
        resp_json = self._send_request(method, url, body_json=body_json, ok_error_codes=ok_error_codes, timeout=timeout)
        if resp_json['type'] == 'async':
            url = '{0}/wait'.format(resp_json['operation'])
            resp_json = self._send_request('GET', url)
            if wait_for_container:
                while resp_json['metadata']['status'] == 'Running':
                    resp_json = self._send_request('GET', url)
            if resp_json['metadata']['status'] != 'Success':
                self._raise_err_from_json(resp_json)
        return resp_json

    def authenticate(self, trust_password):
        body_json = {'type': 'client', 'password': trust_password}
        return self._send_request('POST', '/1.0/certificates', body_json=body_json)

    def _send_request(self, method, url, body_json=None, ok_error_codes=None, timeout=None):
        try:
            body = json.dumps(body_json)
            self.connection.request(method, url, body=body)
            resp = self.connection.getresponse()
            resp_data = resp.read()
            resp_data = to_text(resp_data, errors='surrogate_or_strict')
            resp_json = json.loads(resp_data)
            self.logs.append({
                'type': 'sent request',
                'request': {'method': method, 'url': url, 'json': body_json, 'timeout': timeout},
                'response': {'json': resp_json}
            })
            resp_type = resp_json.get('type', None)
            if resp_type == 'error':
                if ok_error_codes is not None and resp_json['error_code'] in ok_error_codes:
                    return resp_json
                if resp_json['error'] == "Certificate already in trust store":
                    return resp_json
                self._raise_err_from_json(resp_json)
            return resp_json
        except socket.error as e:
            raise LXDClientException('cannot connect to the LXD server', err=e)

    def _raise_err_from_json(self, resp_json):
        err_params = {}
        if self.debug:
            err_params['logs'] = self.logs
        raise LXDClientException(self._get_err_from_resp_json(resp_json), **err_params)

    @staticmethod
    def _get_err_from_resp_json(resp_json):
        err = None
        metadata = resp_json.get('metadata', None)
        if metadata is not None:
            err = metadata.get('err', None)
        if err is None:
            err = resp_json.get('error', None)
        return err


def default_key_file():
    return os.path.expanduser('~/.config/lxc/client.key')


def default_cert_file():
    return os.path.expanduser('~/.config/lxc/client.crt')



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = r"""
---
module: digital_ocean_spaces_info
short_description: List DigitalOcean Spaces.
description:
  - List DigitalOcean Spaces.
author: Mark Mercado (@mamercad)
version_added: 1.15.0
options:
  state:
    description:
     - Only present is supported.
    default: present
    type: str
    choices: ["present"]
  region:
    description:
      - The region from which to list Spaces.
    aliases: ["region_id"]
    required: true
    type: str
  aws_access_key_id:
    description:
      - The AWS_ACCESS_KEY_ID to use.
    required: true
    type: str
    aliases: ["AWS_ACCESS_KEY_ID"]
  aws_secret_access_key:
    description:
      - The AWS_SECRET_ACCESS_KEY to use.
    required: true
    type: str
    aliases: ["AWS_SECRET_ACCESS_KEY"]
requirements:
  - boto3
extends_documentation_fragment:
  - community.digitalocean.digital_ocean.documentation
"""


EXAMPLES = r"""
- name: List all Spaces in nyc3
  community.digitalocean.digital_ocean_spaces_info:
    state: present
    region: nyc3
"""


RETURN = r"""
data:
  description: List of DigitalOcean Spaces
  returned: always
  type: dict
  sample:
    spaces:
      - endpoint_url: https://nyc3.digitaloceanspaces.com
        name: gh-ci-space
        region: nyc3
        space_url: https://gh-ci-space.nyc3.digitaloceanspaces.com
"""

from ansible.module_utils.basic import (
    AnsibleModule,
    missing_required_lib,
    env_fallback,
    to_native,
)
from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import (
    DigitalOceanHelper,
)
from traceback import format_exc

try:
    import boto3

    HAS_BOTO3 = True
except Exception:
    HAS_BOTO3 = False


def run(module):
    state = module.params.get("state")
    region = module.params.get("region")
    aws_access_key_id = module.params.get("aws_access_key_id")
    aws_secret_access_key = module.params.get("aws_secret_access_key")

    if state == "present":
        try:
            session = boto3.session.Session()
            client = session.client(
                "s3",
                region_name=region,
                endpoint_url=f"https://{region}.digitaloceanspaces.com",
                aws_access_key_id=aws_access_key_id,
                aws_secret_access_key=aws_secret_access_key,
            )
            response = client.list_buckets()
        except Exception as e:
            module.fail_json(msg=to_native(e), exception=format_exc())

        response_metadata = response.get("ResponseMetadata")
        http_status_code = response_metadata.get("HTTPStatusCode")

        if http_status_code == 200:
            spaces = [
                {
                    "name": space["Name"],
                    "region": region,
                    "endpoint_url": f"https://{region}.digitaloceanspaces.com",
                    "space_url": f"https://{space['Name']}.{region}.digitaloceanspaces.com",
                }
                for space in response["Buckets"]
            ]
            module.exit_json(changed=False, data={"spaces": spaces})

        module.fail_json(changed=False, msg=f"Failed to list Spaces in {region}")


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    argument_spec.update(
        state=dict(type="str", choices=["present"], default="present"),
        region=dict(type="str", aliases=["region_id"], required=True),
        aws_access_key_id=dict(
            type="str",
            aliases=["AWS_ACCESS_KEY_ID"],
            fallback=(env_fallback, ["AWS_ACCESS_KEY_ID"]),
            required=True,
            no_log=True,
        ),
        aws_secret_access_key=dict(
            type="str",
            aliases=["AWS_SECRET_ACCESS_KEY"],
            fallback=(env_fallback, ["AWS_SECRET_ACCESS_KEY"]),
            required=True,
            no_log=True,
        ),
    )
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    if not HAS_BOTO3:
        module.fail_json(msg=missing_required_lib("boto3"))

    run(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: licensing_smart_state_info
short_description: Information module for Licensing Smart State
description:
- Get all Licensing Smart State.
- Get smart license information.
version_added: '2.1.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options: {}
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Licensing
  description: Complete reference of the Licensing API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!license-openapi
notes:
  - SDK Method used are
    licensing.Licensing.get_smart_state,

  - Paths used are
    get /api/v1/license/system/smart-state,

"""

EXAMPLES = r"""
- name: Get all Licensing Smart State
  cisco.ise.licensing_smart_state_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "connectionType": "string",
      "state": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Ntp_global parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)

from ansible_collections.arista.eos.plugins.module_utils.network.eos.utils.utils import (
    normalize_interface,
)


def _tmplt_ntp_global_serve(config_data):
    el = config_data["serve"]
    command = "ntp serve"
    if el.get("access_lists"):
        command += " {afi} access-group".format(**el["access_lists"])
        if "acls" in el["access_lists"]:
            command += " {acl_name} ".format(**el["access_lists"]["acls"])
            if el["access_lists"]["acls"].get("vrf"):
                command += " vrf {vrf} ".format(**el["access_lists"]["acls"])
            command += el["access_lists"]["acls"]["direction"]
    return command


def _tmplt_ntp_global_authenticate(config_data):
    el = config_data["authenticate"]
    if el.get("enable"):
        command = "ntp authenticate"
        if el.get("servers"):
            command += " servers"
        return command


def _tmplt_ntp_global_authentication_keys(config_data):
    el = config_data["authentication_keys"]
    command = "ntp authentication-key "
    command += str(el["id"])
    command += " " + el["algorithm"]
    if "encryption" in el:
        command += " " + str(el["encryption"])
    if "key" in el:
        command += " " + el["key"]
    return command


def _tmplt_ntp_global_servers(config_data):
    el = config_data["servers"]
    command = "ntp server"
    if el.get("vrf"):
        command += " vrf {vrf}".format(**el)
    if el.get("server"):
        command += " {server}".format(**el)
    if el.get("burst"):
        command += " burst"
    if el.get("iburst"):
        command += " iburst"
    if el.get("key_id"):
        command += " key {key_id}".format(**el)
    if el.get("local_interface"):
        linterface = el.get("local_interface").replace(" ", "")
        command += " local-interface " + linterface
    if el.get("maxpoll"):
        command += " maxpoll {maxpoll}".format(**el)
    if el.get("minpoll"):
        command += " minpoll {minpoll}".format(**el)
    if el.get("prefer"):
        command += " prefer"
    if el.get("source"):
        interface_name = normalize_interface(el["source"])
        command += " source " + interface_name
    if el.get("version"):
        command += " version {version}".format(**el)
    return command


class Ntp_globalTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Ntp_globalTemplate, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "authenticate",
            "getval": re.compile(
                r"""
                \s*ntp\sauthenticate
                \s*(?P<servers>servers)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ntp_global_authenticate,
            "result": {
                "authenticate": {
                    "enable": "{{ True if servers is undefined }}",
                    "servers": "{{ True if servers is defined }}",
                },
            },
        },
        {
            "name": "authentication_keys",
            "getval": re.compile(
                r"""
                \s*ntp\sauthentication-key
                \s+(?P<id>\d+)
                \s+(?P<algo>md5|sha1)
                \s*(?P<enc>0|7)*
                \s+(?P<line>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ntp_global_authentication_keys,
            "result": {
                "authentication_keys": {
                    "{{ id }}": {
                        "id": "{{ id }}",
                        "algorithm": "{{ algo }}",
                        "encryption": "{{ enc }}",
                        "key": "{{ line }}",
                    },
                },
            },
        },
        {
            "name": "local_interface",
            "getval": re.compile(
                r"""
                \s*ntp\slocal-interface
                \s+(?P<int>.+)
                *$""",
                re.VERBOSE,
            ),
            "setval": 'ntp local-interface {{ local_interface }}',
            "result": {
                "local_interface": "{{ int }}",
            },
        },
        {
            "name": "qos_dscp",
            "getval": re.compile(
                r"""
                \s*ntp\sqos\sdscp
                \s+(?P<val>\d+)
                *$""",
                re.VERBOSE,
            ),
            "setval": 'ntp qos dscp {{ qos_dscp }}',
            "result": {
                "qos_dscp": "{{ val }}",
            },
        },
        {
            "name": "serve_all",
            "getval": re.compile(
                r"""
                \s*ntp\sserve
                \s+(?P<all>all)*
                $""",
                re.VERBOSE,
            ),
            "setval": "ntp serve all",
            "compval": "serve",
            "result": {
                "serve": {
                    "all": "{{ True if all is defined }}",
                },
            },
        },
        {
            "name": "serve",
            "getval": re.compile(
                r"""
                \s*ntp\sserve
                \s*(?P<afi>ip|ipv6)*
                \s*(access-group)*
                \s*(?P<name>\S+)*
                \s*(?P<vrf>vrf\s\S+)*
                \s*(?P<dir>in|out)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ntp_global_serve,
            "shared": True,
            "result": {
                "serve": {
                    "access_lists": {
                        "{{ afi }}": {
                            "afi": "{{ afi }}",
                            "acls": [
                                {
                                    "acl_name": "{{ name }}",
                                    "direction": "{{ dir }}",
                                    "vrf": "{{ vrf.split(" ")[1] }}",
                                },
                            ],
                        },
                    },
                },
            },
        },
        {
            "name": "servers",
            "getval": re.compile(
                r"""
                \s*ntp\sserver
                \s*(?P<vrf>vrf\s\S+)*
                \s*(?P<host>\S+)*
                \s*(?P<prefer>prefer)*
                \s*(?P<burst>burst)*
                \s*(?P<iburst>iburst)*
                \s*(?P<maxpoll>maxpoll\s\d+)*
                \s*(?P<minpoll>minpoll\s\d+)*
                \s*(?P<source>source\s.+?)*
                \s*(?P<version>version\s[1-4])*
                \s*(?P<key>key\s.+)*
                (\s*local-interface\s(?P<local_int>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ntp_global_servers,
            "result": {
                "servers": {
                    "{{ host }}": {
                        "vrf": "{{ vrf.split(" ")[1] if vrf is defined }}",
                        "server": "{{ host }}",
                        "burst": "{{ True if burst is defined }}",
                        "iburst": "{{ True if iburst is defined }}",
                        "key_id": "{{ key.split(" ")[1] if key is defined }}",
                        "local_interface": "{{ local_int.replace(' ', '') if local_int is defined }}",
                        "maxpoll": "{{ maxpoll.split(" ")[1] if maxpoll is defined }}",
                        "minpoll": "{{ minpoll.split(" ")[1] if minpoll is defined }}",
                        "source": "{{ source.split(" ")[1] if source is defined }}",
                        "version": "{{ version.split(" ")[1] if version is defined }}",
                        "prefer": "{{ True if prefer is defined }}",
                    },
                },
            },
        },
        {
            "name": "trusted_key",
            "getval": re.compile(
                r"""
                \s*ntp\strusted-key
                \s*(?P<key>.+)*
                $""",
                re.VERBOSE,
            ),
            "setval": 'ntp trusted-key {{ trusted_key }}',
            "result": {
                "trusted_key": "{{ key }}",
            },
        },
    ]
    # fmt: on



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The nxos_logging_global config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from copy import deepcopy

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
    get_from_dict,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.facts import Facts
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.logging_global import (
    Logging_globalTemplate,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.utils.utils import (
    get_logging_sevmap,
)


class Logging_global(ResourceModule):
    """
    The nxos_logging_global config class
    """

    def __init__(self, module):
        super(Logging_global, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="logging_global",
            tmplt=Logging_globalTemplate(),
        )
        self._sev_map = get_logging_sevmap(invert=True)
        self._state_set = ("replaced", "deleted", "overridden")
        self.parsers = [
            "console",
            "module",
            "monitor",
            "logfile",
            "event.link_status.enable",
            "event.link_status.default",
            "event.trunk_status.enable",
            "event.trunk_status.default",
            "history.severity",
            "history.size",
            "ip.access_list.cache.entries",
            "ip.access_list.cache.interval",
            "ip.access_list.cache.threshold",
            "ip.access_list.detailed",
            "ip.access_list.include.sgt",
            "origin_id.hostname",
            "origin_id.ip",
            "origin_id.string",
            "rate_limit",
            "rfc_strict",
            "source_interface",
            "timestamp",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """

        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = self._logging_list_to_dict(self.want)
        haved = self._logging_list_to_dict(self.have)

        if self.state == "deleted":
            # empty out want (in case something was specified)
            # some items are populated later on for correct removal
            wantd = {}

        # pre-process `event.x.y` keys
        for x in self.parsers[4:7]:
            have_k = get_from_dict(haved, x)
            want_k = get_from_dict(wantd, x)
            if have_k is None and want_k is not None:
                # set have to True to mimic default state
                # this allows negate commands to be issued
                self.__update_dict(haved, x)
            if all(
                (
                    self.state in self._state_set,
                    have_k is False,
                    want_k is None,
                ),
            ):
                # if want is missing and have is negated
                # set want to True in order to revert to default state
                self.__update_dict(wantd, x)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            for x in self.parsers[0:4]:
                hstate = haved.get(x, {}).get("state", "")
                wstate = wantd.get(x, {}).get("state", "")
                if hstate == "disabled" and not wstate:
                    # this ensures that updates are done
                    # with correct `state`
                    if wantd.get(x, {}):
                        wantd[x].update({"state": "enabled"})
            wantd = dict_merge(haved, wantd)

        if self.state in self._state_set:
            # set default states for keys that appear in negated form
            for x in self.parsers[0:3]:
                if x in haved and x not in wantd:
                    wantd[x] = {"state": "enabled"}
            if "rate_limit" in haved and "rate_limit" not in wantd:
                wantd["rate_limit"] = "enabled"
            if "logfile" in haved and "logfile" not in wantd:
                wantd["logfile"] = {"name": "messages", "severity": 5}

        self._compare(want=wantd, have=haved)

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Logging_global network resource.
        """
        self.compare(parsers=self.parsers, want=want, have=have)
        self._compare_lists(want, have)

    def _compare_lists(self, want, have):
        """Compare list of dictionaries"""
        for x in ["facilities", "hosts"]:
            wantx = want.get(x, {})
            havex = have.get(x, {})
            for key, wentry in iteritems(wantx):
                hentry = havex.pop(key, {})
                if wentry != hentry:
                    if x == "hosts" and self.state in self._state_set:
                        # remove have config for hosts
                        # else want gets appended
                        self.addcmd(hentry, x, negate=True)
                    self.addcmd(wentry, x)
            for key, hentry in iteritems(havex):
                self.addcmd(hentry, x, negate=True)

    def _logging_list_to_dict(self, data):
        """Convert all list to dicts to dicts
        of dicts and substitute severity values
        """
        tmp = deepcopy(data)
        pkey = {"hosts": "host", "facilities": "facility"}
        for k in ("hosts", "facilities"):
            if k in tmp:
                for x in tmp[k]:
                    if "severity" in x:
                        x["severity"] = self._sev_map[x["severity"]]
                tmp[k] = {i[pkey[k]]: i for i in tmp[k]}
        for k in ("console", "history", "logfile", "module", "monitor"):
            if "severity" in tmp.get(k, {}):
                tmp[k]["severity"] = self._sev_map[tmp[k]["severity"]]
        return tmp

    def __update_dict(self, datadict, key, nval=True):
        """Utility method that updates last subkey of
        `datadict` as identified by `key` to `nval`.
        """
        keys = key.split(".")
        if keys[0] not in datadict:
            datadict[keys[0]] = {}
        if keys[1] not in datadict[keys[0]]:
            datadict[keys[0]][keys[1]] = {}
        datadict[keys[0]][keys[1]].update({keys[2]: nval})



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: native_supplicant_profile
short_description: Resource module for Native Supplicant Profile
description:
- Manage operations update and delete of the resource Native Supplicant Profile.
- This API deletes a native supplicant profile.
- This API allows the client to update a native supplicant profile.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  description:
    description: Native Supplicant Profile's description.
    type: str
  id:
    description: Native Supplicant Profile's id.
    type: str
  name:
    description: Native Supplicant Profile's name.
    type: str
  wirelessProfiles:
    description: Native Supplicant Profile's wirelessProfiles.
    elements: dict
    suboptions:
      actionType:
        description: Action type for WifiProfile. Allowed values - ADD, - UPDATE, -
          DELETE (required for updating existing WirelessProfile).
        type: str
      allowedProtocol:
        description: Native Supplicant Profile's allowedProtocol.
        type: str
      certificateTemplateId:
        description: Native Supplicant Profile's certificateTemplateId.
        type: str
      previousSSID:
        description: Previous ssid for WifiProfile (required for updating existing WirelessProfile).
        type: str
      ssid:
        description: Native Supplicant Profile's ssid.
        type: str
    type: list
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
seealso:
- name: Cisco ISE documentation for NativeSupplicantProfile
  description: Complete reference of the NativeSupplicantProfile API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!nspprofile
notes:
  - SDK Method used are
    native_supplicant_profile.NativeSupplicantProfile.delete_native_supplicant_profile_by_id,
    native_supplicant_profile.NativeSupplicantProfile.update_native_supplicant_profile_by_id,

  - Paths used are
    delete /ers/config/nspprofile/{id},
    put /ers/config/nspprofile/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.ise.native_supplicant_profile:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    description: string
    id: string
    name: string
    wirelessProfiles:
    - actionType: string
      allowedProtocol: string
      certificateTemplateId: string
      previousSsid: string
      ssid: string

- name: Delete by id
  cisco.ise.native_supplicant_profile:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "wirelessProfiles": [
        {
          "ssid": "string",
          "allowedProtocol": "string",
          "certificateTemplateId": "string",
          "actionType": "string",
          "previousSsid": "string"
        }
      ],
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2020 Sakar Mehra (@sakar97), Nikhil Patne (@nikhilpatne)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_apimanagement_info
version_added: "1.6.0"
short_description: Get the infomation of the API Instance
description:
    - Get the information of api instance.
options:
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    service_name:
        description:
            - The name of the API Management service.
        required: true
        type: str
    expand_api_version_set:
        description:
            - Include full ApiVersionSet resource in response
        type: bool
    include_not_tagged_apis:
        description:
            - Included not tagged APIs in the response.
        type: bool
    api_id:
        description:
            - API revision identifier. It must be unique in the current API Management service instance.
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Sakar Mehra (@sakar97)
    - Nikhil Patne (@nikhilpatne)

'''

EXAMPLES = '''
- name: Get the information of api
  azure_rm_apimanagement_info:
    resource_group: myResourceGroup
    service_name: myService
- name: Get the information of api
  azure_rm_apimanagement_info:
    resource_group: myResourceGroup
    service_name: myService
    api_id: testApi
'''

RETURN = '''
api:
    description:
        - A list of dict results where the key is the name of the Api and the values are the facts for that Api.
    returned: always
    type: complex
    contains:
        api_name:
            description:
                - The api name provided by the user.
            type: complex
            contains:
                id:
                    description:
                        - Resource ID.
                    returned: always
                    type: str
                    sample: null
                name:
                    description:
                        - Resource name.
                    returned: always
                    type: str
                    sample: null
                type:
                    description:
                        - Resource type for API Management resource.
                    returned: always
                    type: str
                    sample: null
                properties:
                    description:
                        - Api entity contract properties.
                    returned: always
                    type: dict
                    sample: null

'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
import json


class AzureApiManagementInfo(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            service_name=dict(
                type='str',
                required=True
            ),
            expand_api_version_set=dict(
                type='bool'
            ),
            include_not_tagged_apis=dict(
                type='bool'
            ),
            api_id=dict(
                type='str'
            )
        )

        self.resource_group = None
        self.service_name = None
        self.tags = None
        self.expand_api_version_set = None
        self.include_not_tagged_apis = None
        self.api_id = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200]

        self.query_parameters = {}
        self.query_parameters['api-version'] = '2020-06-01-preview'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        super(AzureApiManagementInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                     supports_check_mode=True,
                                                     supports_tags=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    is_track2=True,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        if (self.resource_group is not None and
                self.service_name is not None and
                self.api_id is not None):
            self.results['api'] = self.get_api_info()
        elif (self.resource_group is not None and
                self.service_name is not None):
            self.results['api'] = self.listbytags()
        elif (self.resource_group is not None and
                self.service_name is not None):
            self.results['api'] = self.listbyservice()
        return self.results

    def get_url(self):
        return '/subscriptions' + '/' + self.subscription_id \
               + '/resourceGroups' + '/' + self.resource_group \
               + '/providers' + '/Microsoft.ApiManagement' + '/service' \
               + '/' + self.service_name + '/apis' + '/' + self.api_id

    def get_url_bytags(self):
        return '/subscriptions' + '/' + self.subscription_id \
               + '/resourceGroups' + '/' + self.resource_group \
               + '/providers' + '/Microsoft.ApiManagement' + '/service' \
               + '/' + self.service_name + '/apisByTags'

    def get_url_byservice(self):
        return '/subscriptions' + '/' + self.subscription_id \
               + '/resourceGroups' + '/' + self.resource_group \
               + '/providers' + '/Microsoft.ApiManagement' + '/service' \
               + '/' + self.service_name + '/apis'

    def get_api_info(self):
        self.url = self.get_url()
        response = None

        try:
            response = self.mgmt_client.query(
                self.url,
                'GET',
                self.query_parameters,
                self.header_parameters,
                None,
                self.status_code,
                600,
                30,
            )
        except Exception as e:
            self.log('Could not get the information.{0}'.format(e))
        try:
            response = json.loads(response.body())
        except Exception:
            return None

        return response

    def listbytags(self):
        self.url = self.get_url_bytags()
        response = None
        try:
            response = self.mgmt_client.query(
                self.url,
                'GET',
                self.query_parameters,
                self.header_parameters,
                None,
                self.status_code,
                600,
                30,
            )
        except Exception as e:
            self.log('Could not get info for the given api tags {0}'.format(e))
        try:
            response = json.loads(response.body())
        except Exception:
            return None

        return response

    def listbyservice(self):
        self.url = self.get_url_byservice()
        response = None
        try:
            response = self.mgmt_client.query(
                self.url,
                'GET',
                self.query_parameters,
                self.header_parameters,
                None,
                self.status_code,
                600,
                30,
            )
            response = json.loads(response.body())
        except Exception as e:
            self.log('Could not get info for a given services.{0}'.format(e))
        try:
            response = json.loads(response.text)
        except Exception:
            return None

        return response


def main():
    AzureApiManagementInfo()


if __name__ == '__main__':
    main()
